package com.softslate.commerce.administrator.core;

import javax.servlet.jsp.JspException;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.struts.taglib.TagUtils;
import org.apache.struts.taglib.html.LinkTag;

/**
 * 
 * @author David Tobey
 */
public class HelpTag extends LinkTag {

	private static final long serialVersionUID = 1L;

	static Log log = LogFactory.getLog(HelpTag.class);

	protected String type = null;

	public String getType() {
		return type;
	}

	public void setType(String type) {
		this.type = type;
	}

	protected String topic = null;

	public String getTopic() {
		return (this.topic);
	}

	public void setTopic(String topic) {
		this.topic = topic;
	}

	protected String title = null;

	public String getTitle() {
		return (this.title);
	}

	public void setTitle(String title) {
		this.title = title;
	}

	public int doStartTag() throws JspException {

		if (getType().equalsIgnoreCase("popup")) {
			return doStartTagPopup();
		}

		return doStartTagInline();

	}

	public int doAfterBody() throws JspException {

		if (getType().equalsIgnoreCase("popup")) {
			return doAfterBodyPopup();
		}

		return doAfterBodyInline();

	}

	public int doEndTag() throws JspException {

		if (getType().equalsIgnoreCase("popup")) {
			return doEndTagPopup();
		}

		return doEndTagInline();

	}

	public int doStartTagInline() throws JspException {

		// Get spaces out of title
		String tokenizedTitle = null;
		if (title != null)
			tokenizedTitle = title.replaceAll(" ", "_");

		// Generate the opening anchor element
		StringBuffer results = new StringBuffer("<a");

		if (target != null) {
			results.append(" target=\"");
			results.append(target);
			results.append("\"");
		}
		if (accesskey != null) {
			results.append(" accesskey=\"");
			results.append(accesskey);
			results.append("\"");
		}
		if (tabindex != null) {
			results.append(" tabindex=\"");
			results.append(tabindex);
			results.append("\"");
		}
		results.append(prepareStyles());
		// results.append(prepareEventHandlers());

		results
				.append(" href=\"#\" class=\"inlineLink inline\" onclick=\"toggleVisibility('");
		if (tokenizedTitle != null) {
			results.append(tokenizedTitle);
		}
		results.append("'); return false;\">");

		if (title != null) {
			results.append(title);
		}

		results.append("</a>");

		if (tokenizedTitle != null) {
			results.append("<div class=\"helpContentInline\" id=\"");
			results.append(tokenizedTitle);
			results.append("\">");
		}

		log.debug(results.toString());

		TagUtils.getInstance().write(pageContext, results.toString());

		// Evaluate the body of this tag
		this.text = null;
		return (EVAL_BODY_INCLUDE);

	}

	public int doAfterBodyInline() throws JspException {

		if (bodyContent != null) {
			String value = bodyContent.getString().trim();
			if (value.length() > 0) {
				text = value;
			}
		}
		return (SKIP_BODY);

	}

	public int doEndTagInline() throws JspException {

		// Prepare the textual content and ending element of this hyperlink
		StringBuffer results = new StringBuffer();
		if (text != null) {
			results.append(text);
		}
		results.append("</div>");

		log.debug(results.toString());
		TagUtils.getInstance().write(pageContext, results.toString());

		return (EVAL_PAGE);

	}

	public int doStartTagPopup() throws JspException {

		// Generate the opening anchor element
		StringBuffer results = new StringBuffer("<a class=\"popup\"");

		if (target != null) {
			results.append(" target=\"");
			results.append(target);
			results.append("\"");
		}
		if (accesskey != null) {
			results.append(" accesskey=\"");
			results.append(accesskey);
			results.append("\"");
		}
		if (tabindex != null) {
			results.append(" tabindex=\"");
			results.append(tabindex);
			results.append("\"");
		}
		results.append(prepareStyles());
		// results.append(prepareEventHandlers());

		// Special case for name anchors
		results.append(" onclick=\"javascript:openHelpPopUp('");
		if (topic != null) {
			results.append(topic);
		}
		results.append("', '");
		if (title != null) {
			results.append("<h3 class=&quot;helpTitle&quot>");
			results.append(title);
			results.append("</h3>");
		}

		log.debug(results.toString());

		TagUtils.getInstance().write(pageContext, results.toString());

		// Evaluate the body of this tag
		this.text = null;
		return (EVAL_BODY_BUFFERED);

	}

	public int doAfterBodyPopup() throws JspException {

		if (bodyContent != null) {
			String value = bodyContent.getString().trim();
			if (value.length() > 0) {
				value = value.replaceAll("'", "&rsquo;");
				value = value.replaceAll("\"", "&quot;");
				value = value.replaceAll("\n", " ");
				value = value.replaceAll("\r", " ");
				text = value;
			}
		}
		return (SKIP_BODY);

	}

	public int doEndTagPopup() throws JspException {

		// Prepare the textual content and ending element of this hyperlink
		StringBuffer results = new StringBuffer();
		if (text != null) {
			results.append(text);
		}
		results.append("');return false;\" href=\"#\">");
		if (title != null) {
			results.append(title);
		}
		results.append("</a>");

		log.debug(results.toString());
		TagUtils.getInstance().write(pageContext, results.toString());

		return (EVAL_PAGE);

	}
}
