package com.softslate.commerce.administrator.core;

import java.util.Iterator;
import java.util.Map;
import java.util.Properties;
import java.util.Set;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.struts.action.ActionForm;
import org.apache.struts.action.ActionForward;
import org.apache.struts.action.ActionMapping;
import org.apache.struts.action.ActionMessage;
import org.apache.struts.action.ActionMessages;

import com.softslate.commerce.customer.core.ActionUtils;
import com.softslate.commerce.customer.core.AppUtils;
import com.softslate.commerce.customer.core.BaseDynaForm;

/**
 * @author David Tobey
 * 
 */
public class SettingsActionComponents extends SettingsAction {

	static Log log = LogFactory.getLog(SettingsActionComponents.class);

	public ActionForward execute(ActionMapping mapping, ActionForm form,
			HttpServletRequest request, HttpServletResponse response)
			throws Exception {
		if (log.isDebugEnabled())
			log.debug("Starting action.");
		BaseDynaForm baseForm = (BaseDynaForm) form;
		synchronized (getServlet().getServletContext()) {

			// Keep track of default, custom, and combined components
			Properties nullProps = null;
			Properties defaultComponents = AppUtils.loadProperties(
					"/appComponents.properties", nullProps);
			Properties customComponents = new Properties();
			Properties appComponents = (Properties) getServlet()
					.getServletContext().getAttribute("appComponents");

			// Only save components whose classes are found and not the default
			String componentsNotFound = "";
			String keysNotSaved = "";
			ActionUtils actionUtils = (ActionUtils) baseForm
					.getBusinessObjectFactory().createUtilsObject(
							"actionUtilsImplementer");
			Map parameters = actionUtils.buildParameterMap(request);
			Set keys = parameters.keySet();
			Iterator i = keys.iterator();
			while (i.hasNext()) {
				String key = (String) i.next();
				if (log.isDebugEnabled())
					log.debug("key: " + key);
				boolean saveThisComponent = true;
				String value = (String) parameters.get(key);

				// Skip the button variable
				if (key.equals("settingsComponentsSubmit")) {
					continue;
				}

				// Skip if the new value is the same as the default
				if (value.equals(defaultComponents.getProperty(key))) {
					if (log.isDebugEnabled())
						log.debug("Skipping component - same as the default: "
								+ key);
					appComponents.setProperty(key, value);
					continue;
				}

				// Check that the new value is a valid class
				if (!key.equals("activePaymentFormTemplates")) {
					String[] valueArray = value.split(",");
					for (int j = 0; j < valueArray.length; j++) {
						String className = valueArray[j].trim();
						try {
							Class.forName(className);
						} catch (ClassNotFoundException cnfe) {
							log.error("Component not found in classpath: "
									+ className, cnfe);
							baseForm
									.getErrors()
									.add(
											ActionMessages.GLOBAL_MESSAGE,
											new ActionMessage(
													"errors.detail",
													"Component "
															+ key
															+ " not saved because "
															+ className
															+ " was not found in the classpath."));
							saveErrors(baseForm.getRequest(),
									(ActionMessages) baseForm.getErrors());
							componentsNotFound = componentsNotFound + className
									+ " ";
							saveThisComponent = false;
						}
					}
				}
				if (saveThisComponent) {
					if (log.isInfoEnabled())
						log.info("Saving component: " + key);
					appComponents.setProperty(key, value);
					customComponents.setProperty(key, value);
				} else {
					if (log.isInfoEnabled())
						log.info("Not saving component: " + key);
					keysNotSaved = keysNotSaved + key + " ";
				}
			}

			// Save the custom components and update the application scope with
			// the combined components.
			saveComponents(customComponents);
			getServlet().getServletContext().setAttribute("appComponents",
					appComponents);
		}
		baseForm.getMessages().add(ActionMessages.GLOBAL_MESSAGE,
				new ActionMessage("messages.settingsMessage"));
		if (!baseForm.getMessages().isEmpty())
			saveMessages(request, baseForm.getMessages());
		return mapping.findForward("success");
	}
}
