package com.softslate.commerce.administrator.customer;

import java.util.Map;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.struts.action.ActionForm;
import org.apache.struts.action.ActionForward;
import org.apache.struts.action.ActionMapping;
import org.apache.struts.action.ActionMessage;
import org.apache.struts.action.ActionMessages;

import com.softslate.commerce.administrator.core.BaseAdministratorForm;
import com.softslate.commerce.businessobjects.customer.Customer;
import com.softslate.commerce.businessobjects.customer.CustomerProcessor;
import com.softslate.commerce.customer.core.BaseAction;

/**
 * Struts action class that corresponds to requests to "/administrator/CustomerEdit.do". Processes a request to edit a
 * record within the system.
 * 
 * @author David Tobey
 * 
 */
public class CustomerEditAction extends BaseAction {

	static Log log = LogFactory.getLog(CustomerEditAction.class);

	public ActionForward execute(ActionMapping mapping, ActionForm form, HttpServletRequest request,
			HttpServletResponse response) throws Exception {
		if (log.isDebugEnabled())
			log.debug("Starting action.");
		BaseAdministratorForm baseForm = (BaseAdministratorForm) form;

		// Edit the record
		CustomerProcessor processor = (CustomerProcessor) baseForm.getBusinessObjectFactory()
				.createObject("customerProcessorImplementer");
		Map results = processor.editCustomer(baseForm.mapOfIncomingParams());

		// Successful update
		if (results == null || results.get("resultCode") == null || results.get("resultCode").equals("0")) {

			// entity.jsp wants the updated record in the request scope
			Customer customer = (Customer) results.get("updatedRecord");
			request.setAttribute("customer", customer);

			// Set the success messages
			baseForm.getMessages().add(ActionMessages.GLOBAL_MESSAGE, new ActionMessage("messages.recordSaved"));
		} else if (results != null && results.get("existingRecord") != null) {
			Customer existingRecord = (Customer) results.get("existingRecord");
			String existingCode = existingRecord.getUserName();
			baseForm.getErrors().add(ActionMessages.GLOBAL_MESSAGE, new ActionMessage(
					"errors.resultCode" + results.get("resultCode"), results.get("duplicateField"), existingCode));
			baseForm.getErrors().add(ActionMessages.GLOBAL_MESSAGE,
					new ActionMessage("errors.editExistingRecord",
							"CustomerEditForm.do?customerID=" + existingRecord.getCustomerID(),
							results.get("duplicateField"), existingCode));
		}

		// Save errors and messages
		saveMessages(baseForm.getRequest(), baseForm.getMessages());
		saveErrors(baseForm.getRequest(), (ActionMessages) baseForm.getErrors());
		return mapping.findForward("success");
	}
}
