package com.softslate.commerce.administrator.email;

import java.util.Map;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.commons.beanutils.PropertyUtils;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.struts.action.ActionForm;
import org.apache.struts.action.ActionForward;
import org.apache.struts.action.ActionMapping;
import org.apache.struts.action.ActionMessage;
import org.apache.struts.action.ActionMessages;

import com.softslate.commerce.administrator.core.BaseAdministratorForm;
import com.softslate.commerce.businessobjects.core.BusinessObjectFactory;
import com.softslate.commerce.businessobjects.email.Email;
import com.softslate.commerce.businessobjects.email.EmailProcessor;
import com.softslate.commerce.businessobjects.email.EmailSender;
import com.softslate.commerce.businessobjects.email.Template;
import com.softslate.commerce.businessobjects.email.TemplateParser;
import com.softslate.commerce.businessobjects.order.Order;
import com.softslate.commerce.customer.core.BaseAction;

/**
 * Struts action class that corresponds to requests to
 * "/administrator/EmailAdd.do". Processes a request to add a record to the
 * system.
 * 
 */
public class EmailAddAction extends BaseAction {

	static Log log = LogFactory.getLog(EmailAddAction.class);

	public ActionForward execute(ActionMapping mapping, ActionForm form, HttpServletRequest request,
			HttpServletResponse response) throws Exception {
		if (log.isDebugEnabled())
			log.debug("Starting action.");

		// Add the record
		BaseAdministratorForm baseForm = (BaseAdministratorForm) form;
		Map parameters = PropertyUtils.describe(form);
		BusinessObjectFactory bof = baseForm.getBusinessObjectFactory();
		EmailProcessor processor = (EmailProcessor) baseForm.getBusinessObjectFactory().createObject(
				"emailProcessorImplementer");
		Map results = processor.addEmail(parameters);

		// Successful insert
		if (results == null || results.get("resultCode") == null || results.get("resultCode").equals("0")) {

			// Set the success messages
			baseForm.getMessages().add(ActionMessages.GLOBAL_MESSAGE, new ActionMessage("messages.recordAdded"));
			if (results != null && results.get("newRecord") != null) {
				Email newRecord = (Email) results.get("newRecord");

				// Set a request scope variable for sub classes to use
				request.setAttribute("newEmailID", new Integer(newRecord.getEmailID()));

				if (parameters.get("action") == null || parameters.get("action").equals("")) {
					baseForm.getMessages().add(
							ActionMessages.GLOBAL_MESSAGE,
							new ActionMessage("messages.editNewRecord", "EmailEditForm.do?emailID="
									+ newRecord.getEmailID()));
				}

				// Optionally send the email
				String action = (String) parameters.get("action");
				if (action != null && (action.equals("Create And Send New Email") || action.endsWith("Send Email"))) {
					sendEmail(bof, newRecord);
					baseForm.getMessages().add(ActionMessages.GLOBAL_MESSAGE,
							new ActionMessage("messages.detail", "Email being sent."));
				}
			}
			saveMessages(baseForm.getRequest(), baseForm.getMessages());

			// If successful, signal the form to be reinstantiated.
			request.setAttribute("reset", "true");
		}

		if (parameters.get("action") != null && !parameters.get("action").equals("") && results != null
				&& results.get("newRecord") != null) {
			Email newRecord = (Email) results.get("newRecord");
			ActionForward af = new ActionForward();
			af.setPath("/EmailEditForm.do?emailID=" + newRecord.getEmailID());
			af.setRedirect(false);
			return af;
		}
		return mapping.findForward("success");
	}

	protected Email createEmailFromOrder(BusinessObjectFactory bof, Order order, String emailType, Template template)
			throws Exception {
		Email email =  bof.createObject(Email.class);
		email.setCustomer(order.getCustomer());
		email.setOrder(order);
		email.setToEmails(order.getEmail1());
		email.setType(emailType);
		email.setTemplate(template);
		return email;
	}

	protected TemplateParser createTemplateParser(BusinessObjectFactory bof, Template template, Email email,
			Map<String, Object> extraTemplateObjects) throws Exception {
		TemplateParser templateParser =  bof.createObject(TemplateParser.class);
		templateParser.initialize(template, email, extraTemplateObjects);
		return templateParser;
	}

	protected void sendEmail(BusinessObjectFactory bof, Email email) throws Exception {
		EmailSender sender =  bof.createObject(EmailSender.class);
		sender.setEmail(email);
		sender.send();
	}

	protected Map<String, Object> createExtraTemplateObjects(Order order) throws Exception {
		return null;
	}
}
