package com.softslate.commerce.businessobjects.administrator;

import java.util.Collection;

import org.hibernate.envers.AuditTable;
import org.hibernate.envers.Audited;

import com.softslate.commerce.businessobjects.core.BusinessObject;

/**
 * Interface representing an administrator within the system.
 * <p>
 * When the application needs to create an instance that implements
 * <code>sscAdministrator</code>,
 * {@link com.softslate.commerce.businessobjects.core.BusinessObjectFactory}
 * finds the name of the Java class to instantiate from the
 * "administratorImplementer" setting in the
 * <code>appComponents.properties</code> file.
 * <p>
 * The default "administratorImplementer" is
 * {@link com.softslate.commerce.businessobjects.administrator.AdministratorBean}.
 * 
 * @author David Tobey
 */
@Audited
@AuditTable("AudAdministrator")
public interface Administrator extends BusinessObject {

	abstract int getAdministratorID();

	abstract void setAdministratorID(int administratorID);

	abstract String getUserName();

	abstract void setUserName(String userName);

	abstract String getPassword();

	abstract void setPassword(String password);

	abstract String getDecryptedPassword();

	abstract void setDecryptedPassword(String decryptedPassword);

	abstract String getLastLogin();

	abstract void setLastLogin(String lastLogin);
	
	abstract boolean getIsApiUser();

	abstract void setIsApiUser(boolean apiUser);
	
	abstract String getPasswordExpires();

	abstract void setPasswordExpires(String passwordExpires);
	
	abstract Integer getNumberOfFailedLogins();

	abstract void setNumberOfFailedLogins(Integer numberOfFailedLogins);
	
	abstract String getLockedOutUntil();

	abstract void setLockedOutUntil(String lockedOutUntil);
	
	abstract String getPreviousPasswords();

	abstract void setPreviousPasswords(String previousPasswords);

	abstract String getAllowedIPAddresses();

	abstract void setAllowedIPAddresses(String allowedIPAddresses);

	abstract boolean getIsActive();

	abstract void setIsActive(boolean isActive);

	abstract Collection getRoleCodes();

	abstract void setRoleCodes(Collection roleCodes);

	abstract int getImportID();

	abstract void setImportID(int importID);

	abstract Collection getExportedAttributeIDs();

	abstract void setExportedAttributeIDs(Collection exportedAttributeIDs);

	abstract Collection getAdministratorRoles();

	abstract void setAdministratorRoles(Collection administratorRoles);
}
