package com.softslate.commerce.businessobjects.events;

import java.util.HashMap;
import java.util.Map;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import com.google.common.eventbus.Subscribe;
import com.softslate.commerce.businessobjects.core.Settings;
import com.softslate.commerce.businessobjects.email.Email;
import com.softslate.commerce.businessobjects.email.TemplateProcessor;
import com.softslate.commerce.businessobjects.order.Order;
import com.softslate.commerce.customer.core.ActionUtils;
import com.softslate.commerce.daos.order.OrderDAO;

public class OrderStatusEmailNotificationTriggerImpl extends EmailNotificationTrigger implements
		OrderStatusEmailNotificationTrigger {

	static Log log = LogFactory.getLog(OrderStatusEmailNotificationTriggerImpl.class);

	protected Order order;
	protected Map originalParameters;
	protected Settings settings;
	protected ActionUtils actionUtils;

	public boolean getExecuteInSeparateThread() {
		return true;
	}


	@Subscribe
	public void processOrderStatusChange(ChangedOrderEvent coe) throws Exception {
		log.info("Trigger for order status change email notification.");
		templateCode = null;
		emailType = null;
		initializeFromEvent(coe);
		Order order = coe.getOrder();
		Map originalParameters = coe.getOriginalParameters();
		if (order != null && order.getStatus() != null && originalParameters != null
				&& !order.getStatus().equals(originalParameters.get("status"))) {
			log.info("Processing order status change email for order being changed to " + order.getStatus()
					+ ". Order number: " + order.getOrderNumber());

			// Calls processTrigger, after BaseTrigger optionally creates
			// a new thread or sets up exceptions to be swallowed.
			process();
		}
	}

	public void initializeFromEvent(ChangedOrderEvent coe) throws Exception {
		order = coe.getOrder();
		bof = coe.getBusinessProcessor().getBusinessObjectFactory();
		bof.getDaoFactory().createDAO(OrderDAO.class).initializeOrder(order);
		templateProcessor = bof.createObject(TemplateProcessor.class);
		actionUtils = bof.createUtilsObject(ActionUtils.class);
		originalParameters = coe.getOriginalParameters();
		settings = coe.getBusinessProcessor().getSettings();
	}

	protected Email createEmail() throws Exception {
		return createEmailFromOrder(order);
	}

	protected Map<String, Object> createExtraTemplateObjects() throws Exception {
		return createExtraTemplateObjects(order);
	}

	protected Map<String, Object> createExtraTemplateObjects(Order order) throws Exception {
		Map<String, Object> extraTemplateObjects = new HashMap<String, Object>();
		String linkedTrackingNumber = actionUtils.createTrackingNumberLink(order.getTrackingNumber());
		extraTemplateObjects.put("linkedTrackingNumber", linkedTrackingNumber);
		return extraTemplateObjects;
	}

	protected void initializeTemplateCodeAndType() throws Exception {
		if ("Shipped".equals(order.getStatus())) {
			templateCode = settings.getValue("OrderShippedJob-templateCode");
			emailType = settings.getValue("OrderShippedJob-emailType");
			log.info("Sending notification for Shipped status using template: " + templateCode);
		}
	}
}
