package com.softslate.commerce.businessobjects.order;

import java.util.HashMap;
import java.util.Map;

import org.apache.commons.beanutils.BeanUtils;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import com.softslate.commerce.businessobjects.core.BaseBusinessProcessor;
import com.softslate.commerce.daos.order.StateDAO;
import com.softslate.commerce.daos.order.StateGatewayDAO;

/**
 * Processes business logic having to do with states. Created in the Struts
 * layer in response to requests for viewing, editing, adding, and deleting
 * states.
 * <p>
 * <code>BasicStateProcessor</code> is the default implementation of the
 * <code>StateProcessor</code> interface for the application.
 * 
 * @author David Tobey
 * 
 */
public class BasicStateProcessor extends BaseBusinessProcessor implements
        StateProcessor {

    static Log log = LogFactory.getLog(BasicStateProcessor.class);

    public Map loadStatesAndCount(Map parameters) throws Exception {
        if (log.isInfoEnabled() && getUser().getAdministrator() != null)
            log.info(getUser().getAdministrator().getUserName()
                    + ": Loading states and count");
        StateGatewayDAO gatewayDAO =  getDaoFactory()
                .createDAO(StateGatewayDAO.class);
        return gatewayDAO.loadStatesAndCount(parameters);
    }

    public Map addState(Map parameters) throws Exception {
        if (log.isInfoEnabled() && getUser().getAdministrator() != null) {
            String code = (String) parameters.get("code");
            log.info(getUser().getAdministrator().getUserName()
                    + ": Adding state: " + code);
        }
		Map results = new HashMap();
        State state = (State) getBusinessObjectFactory().createObject(
                "stateImplementer");
        BeanUtils.copyProperties(state, parameters);
        Country country = (Country) getBusinessObjectFactory().createObject(
                "countryImplementer");
        country.setCountryID(state.getCountryID());
        state.setCountry(country);
        StateDAO dao = (StateDAO) getDaoFactory().createDAO(
                "stateDAOImplementer");
        dao.setState(state);
        dao.insertState(true);
		results.put("newRecord", dao.getState());
		return results;
    }

    public State loadStateFromID(Map parameters) throws Exception {
        if (log.isInfoEnabled() && getUser().getAdministrator() != null)
            log.info(getUser().getAdministrator().getUserName()
                    + ": Loading state from id");
        State state = (State) getBusinessObjectFactory().createObject(
                "stateImplementer");
        StateDAO dao = (StateDAO) getDaoFactory().createDAO(
                "stateDAOImplementer");
        BeanUtils.copyProperties(state, parameters);
        dao.setState(state);
        dao.loadStateFromID();
		return dao.getState();
    }

    public Map editState(Map parameters) throws Exception {
        if (log.isInfoEnabled() && getUser().getAdministrator() != null) {
            String code = (String) parameters.get("code");
            log.info(getUser().getAdministrator().getUserName()
                    + ": Editing state: " + code);
        }
		Map results = new HashMap();
		State state = loadStateFromID(parameters);
        BeanUtils.copyProperties(state, parameters);
        Country country = (Country) getBusinessObjectFactory().createObject(
                "countryImplementer");
        country.setCountryID(state.getCountryID());
        state.setCountry(country);
        StateDAO dao = (StateDAO) getDaoFactory().createDAO(
                "stateDAOImplementer");
        dao.setState(state);
        dao.updateState(true);
		results.put("updatedRecord", dao.getState());
		return results;
    }

    public Map editDeleteStates(Map parameters) throws Exception {
        if (log.isInfoEnabled() && getUser().getAdministrator() != null)
            log.info(getUser().getAdministrator().getUserName()
                    + ": Editing/deleting multiple states");
		String[] idValues = (String[]) parameters.get("idValues");
		String[] displayFields = (String[]) parameters.get("displayFields");
		Country[] country = new Country[idValues.length];
		for (int i = 0; i < idValues.length; i++) {
			country[i] = null;
			for (int j = 0; j < displayFields.length; j++) {
				String displayField = displayFields[j];
				if (displayField.equals("countryID")
						|| displayField.equals("country")) {
					displayFields[j] = "country";
					String[] fieldValues = (String[]) parameters
							.get("countryID");
					if (fieldValues != null && fieldValues[i] != null
							&& !fieldValues[i].equals("0")) {
						if (log.isDebugEnabled())
							log.debug("Creating country of "
									+ fieldValues[i] + " for id: "
									+ idValues[i]);
						Country object =  getBusinessObjectFactory()
								.createObject(Country.class);
						object.setCountryID((new Integer(fieldValues[i]))
								.intValue());
						country[i] = object;
					}
				}
			}
		}
		parameters.put("country", country);
        StateGatewayDAO dao = (StateGatewayDAO) getDaoFactory().createDAO(
                "stateGatewayDAOImplementer");
		Map results = dao.editDeleteStates(parameters, getBusinessObjectFactory());
		return results;
    }

    public void deleteState(Map parameters) throws Exception {
        if (log.isInfoEnabled() && getUser().getAdministrator() != null) {
            String code = (String) parameters.get("code");
            log.info(getUser().getAdministrator().getUserName()
                    + ": Deleting state: " + code);
        }
        State state = (State) getBusinessObjectFactory().createObject(
                "stateImplementer");
        BeanUtils.copyProperties(state, parameters);
        StateDAO dao = (StateDAO) getDaoFactory().createDAO(
                "stateDAOImplementer");
        dao.setState(state);
        dao.deleteState(true);
        return;
    }
}
