package com.softslate.commerce.businessobjects.order;

import java.io.Serializable;
import java.util.Collection;

import org.hibernate.envers.AuditTable;
import org.hibernate.envers.Audited;

import com.softslate.commerce.businessobjects.core.BaseBusinessObject;

/**
 * Object representing a country within the system. <code>CountryBean</code>
 * is the default implementation of the <code>Country</code> interface for the
 * application.
 * <p>
 * The database stores countries in the <code>sscCountry</code> table, and
 * this interface can be used to represent a single row of that table. The
 * Hibernate configuration file <code>CountryBean.hbm.xml</code> maps that
 * table's columns to the properties in this object.
 * 
 * @author David Tobey
 * 
 */
@Audited
@AuditTable("AudCountry")
public class CountryBean extends BaseBusinessObject implements Country,
		Serializable {

	private static final long serialVersionUID = 5980324936256812630L;

	private int countryID = 0;

	private String code = null;

	private String name = null;

	private int countryOrder = 0;

	private boolean allowOtherStates = false;

	private boolean isActive = true;

	private Collection states = null;

	public boolean getAllowOtherStates() {
		return allowOtherStates;
	}

	public void setAllowOtherStates(boolean allowOtherStates) {
		this.allowOtherStates = allowOtherStates;
	}

	public String getCode() {
		return code;
	}

	public void setCode(String code) {
		this.code = code;
	}

	public int getCountryOrder() {
		return countryOrder;
	}

	public void setCountryOrder(int countryOrder) {
		this.countryOrder = countryOrder;
	}

	public boolean getIsActive() {
		return isActive;
	}

	public void setIsActive(boolean isActive) {
		this.isActive = isActive;
	}

	public String getName() {
		return name;
	}

	public void setName(String name) {
		this.name = name;
	}

	public int getCountryID() {
		return countryID;
	}

	public void setCountryID(int countryID) {
		this.countryID = countryID;
	}

	public Collection getStates() {
		return states;
	}

	public void setStates(Collection states) {
		this.states = states;
	}

}
