package com.softslate.commerce.businessobjects.order;

import com.softslate.commerce.businessobjects.core.BusinessObject;

/**
 * Interface representing an order number within the system. An order number is
 * generated as each order is placed. This interface helps ensure that each
 * order number is unique, and that order numbers go upward sequentially with no
 * gaps.
 * <p>
 * When the application needs to create an instance that implements
 * <code>OrderNumber/code>,
 * {@link com.softslate.commerce.businessobjects.core.BusinessObjectFactory}
 * finds the name of the Java class to instantiate from the "orderNumberImplementer"
 * setting in the <code>appComponents.properties</code> file.
 * <p>
 * The default "orderNumberImplementer" is
 * {@link com.softslate.commerce.businessobjects.order.OrderNumberBean}.
 * 
 * @author David Tobey
 * 
 * 
 */
public interface OrderNumber extends BusinessObject {

	abstract int getOrderNumber();

	abstract void setOrderNumber(int orderNumber);

}
