package com.softslate.commerce.businessobjects.product;

import java.io.Serializable;
import java.util.Collection;
import java.util.Map;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.hibernate.envers.AuditTable;
import org.hibernate.envers.Audited;
import org.hibernate.envers.NotAudited;

import com.softslate.commerce.businessobjects.core.BaseBusinessObject;

/**
 * Object representing a category within the system. <code>CategoryBean</code>
 * is the default implementation of the <code>Category</code> interface for
 * the application.
 * <p>
 * The database stores categories in the <code>sscCategory</code> table, and
 * this interface can be used to represent a single row of that table. The
 * Hibernate configuration file <code>CategoryBean.hbm.xml</code> maps that
 * table's columns to the properties in this object.
 * 
 * @author David Tobey
 */
@Audited
@AuditTable("AudCategory")
public class CategoryBean extends BaseBusinessObject implements Category,
		Serializable {

	static Log log = LogFactory.getLog(CategoryBean.class);

	private static final long serialVersionUID = -8246461262360472328L;

	private int categoryID = 0;

	private int categoryLevel = 0;

	private int categoryOrder = 0;

	private Map categorySettings;

	private int categoryTreeID = 1;

	private String code = null;

	private String description = null;

	private String extra1 = null;

	private String extra2 = null;

	private String extra3 = null;

	private int firstRow = 1;

	private String footer = null;

	private String header = null;

	private boolean isActive = true;

	private boolean isContentPage = false;

	private int itemCount = 0;

	private int itemsPerPage = 10;

	private String keywords = null;

	private String largeImage;

	private String mediumImage;

	private String name = null;

	private Category parent = null;

	private Integer parentCategory = null;

	private Collection parentCollection = null;

	private String parentString = null;

	private Collection productCategories = null;

	private Collection productListCollection = null;

	private Collection products = null;

	private String smallImage;

	private Collection subcategoryCollection = null;

	public int getCategoryID() {
		return (this.categoryID);
	}

	public int getCategoryLevel() {
		return categoryLevel;
	}

	public int getCategoryOrder() {
		return (this.categoryOrder);
	}

	public Map getCategorySettings() {
		return categorySettings;
	}

	public int getCategoryTreeID() {
		return categoryTreeID;
	}

	public String getCode() {
		return (this.code);
	}

	public String getDescription() {
		return (this.description);
	}

	public String getExtra1() {
		return extra1;
	}

	public String getExtra2() {
		return extra2;
	}

	public String getExtra3() {
		return extra3;
	}

	public int getFirstRow() {
		return (this.firstRow);
	}

	public String getFooter() {
		return (this.footer);
	}

	public String getHeader() {
		return (this.header);
	}

	public boolean getIsActive() {
		return (this.isActive);
	}

	public int getItemCount() {
		return (this.itemCount);
	}

	public int getItemsPerPage() {
		return (this.itemsPerPage);
	}

	public String getKeywords() {
		return keywords;
	}

	public String getLargeImage() {
		return (this.largeImage);
	}

	public String getMediumImage() {
		return (this.mediumImage);
	}

	public String getName() {
		return (this.name);
	}

	public Category getParent() {
		return (this.parent);
	}

	public Integer getParentCategory() {
		return (this.parentCategory);
	}

	public Collection getParentCollection() {
		return (this.parentCollection);
	}

	public String getParentString() {
		return (this.parentString);
	}

	public Collection getProductCategories() {
		return (this.productCategories);
	}

	public Collection getProductListCollection() {
		return (this.productListCollection);
	}

	@NotAudited
	public Collection getProducts() {
		return products;
	}

	public String getSmallImage() {
		return (this.smallImage);
	}

	@NotAudited
	public Collection getSubcategoryCollection() {
		return (this.subcategoryCollection);
	}

	public void setCategoryID(int categoryID) {
		this.categoryID = categoryID;
	}

	public void setCategoryLevel(int categoryLevel) {
		this.categoryLevel = categoryLevel;
	}

	public void setCategoryOrder(int categoryOrder) {
		this.categoryOrder = categoryOrder;
	}

	public void setCategorySettings(Map categorySettings) {
		this.categorySettings = categorySettings;
	}

	public void setCategoryTreeID(int categoryTreeID) {
		this.categoryTreeID = categoryTreeID;
	}

	public void setCode(String code) {
		this.code = code;
	}

	public void setDescription(String description) {
		this.description = description;
	}

	public void setExtra1(String extra1) {
		this.extra1 = extra1;
	}

	public void setExtra2(String extra2) {
		this.extra2 = extra2;
	}

	public void setExtra3(String extra3) {
		this.extra3 = extra3;
	}

	public void setFirstRow(int firstRow) {
		this.firstRow = firstRow;
	}

	public void setFooter(String footer) {
		this.footer = footer;
	}

	public void setHeader(String header) {
		this.header = header;
	}

	public void setIsActive(boolean isActive) {
		this.isActive = isActive;
	}

	public void setItemCount(int itemCount) {
		this.itemCount = itemCount;
	}

	public void setItemsPerPage(int itemsPerPage) {
		this.itemsPerPage = itemsPerPage;
	}

	public void setKeywords(String keywords) {
		this.keywords = keywords;
	}

	public void setLargeImage(String largeImage) {
		this.largeImage = largeImage;
	}

	public void setMediumImage(String mediumImage) {
		this.mediumImage = mediumImage;
	}

	public void setName(String name) {
		this.name = name;
	}

	public void setParent(Category parent) {
		this.parent = parent;
	}

	public void setParentCategory(Integer parentCategory) {
		this.parentCategory = parentCategory;
	}

	public void setParentCollection(Collection parentCollection) {
		this.parentCollection = parentCollection;
	}

	public void setParentString(String parentString) {
		this.parentString = parentString;
	}

	public void setProductCategories(Collection productCategories) {
		this.productCategories = productCategories;
	}

	public void setProductListCollection(Collection productListCollection) {
		this.productListCollection = productListCollection;
	}

	public void setProducts(Collection products) {
		this.products = products;
	}

	public void setSmallImage(String smallImage) {
		this.smallImage = smallImage;
	}

	public void setSubcategoryCollection(Collection subcategoryCollection) {
		this.subcategoryCollection = subcategoryCollection;
	}

	public boolean getIsContentPage() {
		return isContentPage;
	}

	public void setIsContentPage(boolean isContentPage) {
		this.isContentPage = isContentPage;
	}

}
