package com.softslate.commerce.customer.customer;

import java.io.File;
import java.util.Collection;
import java.util.Iterator;
import java.util.Map;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.struts.action.ActionForm;
import org.apache.struts.action.ActionForward;
import org.apache.struts.action.ActionMapping;
import org.apache.struts.action.ActionMessage;
import org.apache.struts.action.ActionMessages;

import com.softslate.commerce.customer.core.BaseAction;
import com.softslate.commerce.businessobjects.customer.CustomerProcessor;

/**
 * Struts action class used for requests to download a file. Used for requests
 * to "/ProcessDownload.do".
 * <p>
 * The <code>execute</code> method of this class uses
 * {@link com.softslate.commerce.businessobjects.customer.CustomerProcessor#loadDownloadFiles()}
 * to load the file downloads that correspond to the products the customer has
 * prevously ordered. It loops through these downloads to find the one
 * corresponding to the <code>productSettingID</code> that was submitted. If
 * successful, a <code>java.io.File</code> object is created from the file's
 * path on the server and a check is made to make sure the file exists. Then
 * request attributes named "file", "name", and "mimeType" are set with the
 * values of the <code>java.io.File</code> object, the name of the file, and
 * mime type of the file, respectively.
 * 
 * @author David Tobey
 * 
 */
public class ProcessDownloadAction extends BaseAction {
	static Log log = LogFactory.getLog(ProcessDownloadAction.class);

	public ActionForward execute(ActionMapping mapping, ActionForm form,
			HttpServletRequest request, HttpServletResponse response)
			throws Exception {
		if (log.isInfoEnabled())
			log.info("Starting action.");
		DownloadForm baseForm = (DownloadForm) form;
		synchronized (baseForm.getRequest().getSession()) {

			// Preserve the original request so the customer is redirected
			// back to the main downloads page after logging in
			request.setAttribute("originalRequest", "/AccountDownload.do");
			
			// If the user is not logged in, it's an error.
			if (baseForm.getUser().getCustomer() == null) {
				baseForm.getErrors().add(ActionMessages.GLOBAL_MESSAGE,
						new ActionMessage("errors.userNotLoggedIn"));
				saveErrors(baseForm.getRequest(), (ActionMessages) baseForm
						.getErrors());
				if (log.isInfoEnabled() && !baseForm.getErrors().isEmpty())
					log.info("Validation error(s): "
							+ baseForm.getErrors().toString());
				return mapping.findForward("notLoggedIn");
			}
			// Query for the customer's available downloads
			CustomerProcessor customerProcessor = (CustomerProcessor) baseForm
					.getBusinessObjectFactory().createObject(
							"customerProcessorImplementer");
			Collection downloads = customerProcessor.loadDownloadFiles();
			baseForm.getRequest().setAttribute("downloads", downloads);
			// Validate the request and find the file to be downloaded.
			String filePath = null;
			String mimeType = null;
			boolean fileValid = false;
			Iterator i = downloads.iterator();
			while (i.hasNext()) {
				Map download = (Map) i.next();
				if (new Integer(baseForm.getProductSettingID())
						.equals((Integer) download.get("productSettingID"))) {
					String names = (String) download.get("downloadsNames");
					String files = (String) download.get("downloadsFiles");
					String mimes = (String) download.get("downloadsMimes");
					if (names != null && files != null && mimes != null) {
						String[] namesArray = names.split(",");
						String[] filesArray = files.split(",");
						String[] mimesArray = mimes.split(",");
						for (int j = 0; j < namesArray.length; j++) {
							if (baseForm.getName().equals(namesArray[j])) {
								fileValid = true;
								filePath = filesArray[j];
								mimeType = mimesArray[j];
								break;
							}
						}
						if (fileValid) {
							break;
						}
					}
				}
			}
			// Add an error if the request is not valid
			if (!fileValid) {
				baseForm.getErrors().add(
						ActionMessages.GLOBAL_MESSAGE,
						new ActionMessage("errors.fileNotValid", baseForm
								.getName()));
				saveErrors(baseForm.getRequest(), (ActionMessages) baseForm
						.getErrors());
				if (log.isInfoEnabled() && !baseForm.getErrors().isEmpty())
					log.info("Validation error(s): "
							+ baseForm.getErrors().toString());
				return mapping.findForward("failure");
			}
			// Find the file - error if it doesn't exist
			File file = new File(filePath);
			if (!file.exists()) {
				baseForm.getErrors().add(
						ActionMessages.GLOBAL_MESSAGE,
						new ActionMessage("errors.fileNotFound", baseForm
								.getName()));
				saveErrors(baseForm.getRequest(), (ActionMessages) baseForm
						.getErrors());
				if (log.isInfoEnabled() && !baseForm.getErrors().isEmpty())
					log.info("Validation error(s): "
							+ baseForm.getErrors().toString());
				return mapping.findForward("failure");
			}
			// Put the File, the file name, and the mime type in the request
			// scope
			if (log.isInfoEnabled() && !baseForm.getErrors().isEmpty())
				log.info("Customer "
						+ baseForm.getUser().getCustomer().getUserName()
						+ " is downloading " + baseForm.getName());
			baseForm.getRequest().setAttribute("file", file);
			baseForm.getRequest().setAttribute("name", baseForm.getName());
			baseForm.getRequest().setAttribute("mimeType", mimeType);
			return mapping.findForward("success");
		}
	}
}
