package com.softslate.commerce.customer.order;

import java.lang.reflect.Method;

import javax.servlet.http.HttpServletRequest;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.struts.action.ActionErrors;
import org.apache.struts.action.ActionMapping;
import org.apache.struts.action.ActionMessage;
import org.apache.struts.action.ActionMessages;
import org.apache.struts.validator.Resources;

import com.softslate.commerce.businessobjects.customer.Customer;
import com.softslate.commerce.businessobjects.order.Order;

/**
 * Struts form class representing a user's billing and delivery addresses. It is
 * also a superclass of
 * {@link com.softslate.commerce.customer.order.CheckoutAddressesForm}, which
 * is used for submissions to "/CheckoutAddresses.do", where both billing and
 * delivery addresses, as well as a shipping option, are submitted.
 * <p>
 * <code>AddressesForm</code> is a subclass of
 * {@link com.softslate.commerce.customer.order.BillingAddressForm}, and it
 * inherits its billing address properties.
 * <p>
 * The <code>validate</code> method of this class first checks to see if the
 * user chose to use the same address for billing and delivery. If so, it copies
 * the billing address properties to the delivery properties. It then loops
 * through the "deliveryRequired" database setting and makes sure each required
 * field is populated. It also checks to make sure the submitted delivery state
 * or province corresponds to the submitted country, and that fields such as
 * emails and phone numbers are in a valid format.
 * 
 * @author David Tobey
 * 
 */
public class AddressesForm extends BillingAddressForm {
	private static final long serialVersionUID = 3257282552254444595L;

	static Log log = LogFactory.getLog(AddressesForm.class);

	private boolean deliveryUseBilling = false;

	private String deliveryFirstName = null;

	private String deliveryLastName = null;

	private String deliveryOrganization = null;

	private String deliveryAddress1 = null;

	private String deliveryAddress2 = null;

	private String deliveryCity = null;

	private String deliveryState = null;

	private String deliveryOther = null;

	private String deliveryPostalCode = null;

	private String deliveryCountry = null;

	private String deliveryPhone1 = null;

	private String deliveryPhone2 = null;

	private String deliveryEmail1 = null;

	private String deliveryEmail2 = null;

	private String deliveryExtra1 = null;

	private String deliveryExtra2 = null;

	private String deliveryExtra3 = null;

	private String deliveryExtra4 = null;

	private String deliveryExtra5 = null;

	private String deliveryNotes = null;

	public boolean getDeliveryUseBilling() {
		return deliveryUseBilling;
	}

	public String getDeliveryAddress1() {
		return deliveryAddress1;
	}

	public String getDeliveryAddress2() {
		return deliveryAddress2;
	}

	public String getDeliveryCity() {
		return deliveryCity;
	}

	public String getDeliveryCountry() {
		return deliveryCountry;
	}

	public String getDeliveryEmail1() {
		return deliveryEmail1;
	}

	public String getDeliveryEmail2() {
		return deliveryEmail2;
	}

	public String getDeliveryExtra1() {
		return deliveryExtra1;
	}

	public String getDeliveryExtra2() {
		return deliveryExtra2;
	}

	public String getDeliveryFirstName() {
		return deliveryFirstName;
	}

	public String getDeliveryLastName() {
		return deliveryLastName;
	}

	public String getDeliveryNotes() {
		return deliveryNotes;
	}

	public String getDeliveryOrganization() {
		return deliveryOrganization;
	}

	public String getDeliveryPhone1() {
		return deliveryPhone1;
	}

	public String getDeliveryPhone2() {
		return deliveryPhone2;
	}

	public String getDeliveryPostalCode() {
		return deliveryPostalCode;
	}

	public String getDeliveryState() {
		return deliveryState;
	}

	public String getDeliveryOther() {
		return deliveryOther;
	}

	public void setDeliveryUseBilling(boolean b) {
		deliveryUseBilling = b;
	}

	public void setDeliveryAddress1(String string) {
		deliveryAddress1 = string;
	}

	public void setDeliveryAddress2(String string) {
		deliveryAddress2 = string;
	}

	public void setDeliveryCity(String string) {
		deliveryCity = string;
	}

	public void setDeliveryCountry(String string) {
		deliveryCountry = string;
	}

	public void setDeliveryEmail1(String string) {
		deliveryEmail1 = string;
	}

	public void setDeliveryEmail2(String string) {
		deliveryEmail2 = string;
	}

	public void setDeliveryExtra1(String string) {
		deliveryExtra1 = string;
	}

	public void setDeliveryExtra2(String string) {
		deliveryExtra2 = string;
	}

	public String getDeliveryExtra3() {
		return deliveryExtra3;
	}

	public void setDeliveryExtra3(String deliveryExtra3) {
		this.deliveryExtra3 = deliveryExtra3;
	}

	public String getDeliveryExtra4() {
		return deliveryExtra4;
	}

	public void setDeliveryExtra4(String deliveryExtra4) {
		this.deliveryExtra4 = deliveryExtra4;
	}

	public String getDeliveryExtra5() {
		return deliveryExtra5;
	}

	public void setDeliveryExtra5(String deliveryExtra5) {
		this.deliveryExtra5 = deliveryExtra5;
	}

	public void setDeliveryFirstName(String string) {
		deliveryFirstName = string;
	}

	public void setDeliveryLastName(String string) {
		deliveryLastName = string;
	}

	public void setDeliveryNotes(String string) {
		deliveryNotes = string;
	}

	public void setDeliveryOrganization(String string) {
		deliveryOrganization = string;
	}

	public void setDeliveryPhone1(String string) {
		deliveryPhone1 = string;
	}

	public void setDeliveryPhone2(String string) {
		deliveryPhone2 = string;
	}

	public void setDeliveryPostalCode(String string) {
		deliveryPostalCode = string;
	}

	public void setDeliveryState(String string) {
		deliveryState = string;
	}

	public void setDeliveryOther(String string) {
		deliveryOther = string;
	}

	public ActionErrors validate(ActionMapping mapping,
			HttpServletRequest request) {
		if (log.isInfoEnabled())
			log.info("Starting validation.");

		super.validate(mapping, request);
		copyBillingToDelivery();
		validateDeliveryFields();

		if (log.isDebugEnabled())
			log.debug("Finished validation.");
		if (log.isInfoEnabled() && !getErrors().isEmpty())
			log.info("Validation error(s): " + getErrors().toString());
		return getErrors();
	}

	/**
	 * If this forms deliveryUseBilling property is set, copies billing
	 * properties to delivery properties.
	 * 
	 */
	public void copyBillingToDelivery() {
		// Set up a Class for reflection
		Class classDefinition = this.getClass();
		Class[] argumentTypes = null;
		Object[] arguments = null;
		// If the 'deliver to billing address' box was checked, copy
		// the billing fields into the delivery fields.
		if (getDeliveryUseBilling()) {
			String billingPrompt = "";
			String deliveryPrompt = "";
			try {
				billingPrompt = (String) getSettings()
						.getValue("billingPrompt");
				deliveryPrompt = (String) getSettings().getValue(
						"deliveryPrompt");
			} catch (Exception e) {
				deliveryPrompt = "";
			}
			String[] billingPromptArray = billingPrompt.split(",");
			for (int i = 0; i < billingPromptArray.length; i++) {
				// Set up a Class for reflection
				String thisFieldString = billingPromptArray[i];
				// Only copy over those that were prompted in the
				// delivery address form in the first place
				if (deliveryPrompt.indexOf(thisFieldString) != -1) {
					String billingMethodString = "getBilling"
							+ formUtils().capitalize(thisFieldString);
					String deliveryMethodString = "setDelivery"
							+ formUtils().capitalize(thisFieldString);
					if (log.isDebugEnabled())
						log.debug("Copying: " + billingMethodString + " to: "
								+ deliveryMethodString);
					String thisBillingValue = null;
					try {
						Method method = classDefinition.getMethod(
								billingMethodString, argumentTypes);
						thisBillingValue = (String) method.invoke(this,
								arguments);
						Class[] argumentTypes2 = { String.class };
						Object[] arguments2 = { thisBillingValue };
						Method method2 = classDefinition.getMethod(
								deliveryMethodString, argumentTypes2);
						method2.invoke(this, arguments2);
					} catch (Exception e) {
						log.error("Copy failed: " + e.toString(), e);
					}
				}
			}
		}
		return;
	}

	/**
	 * Validates this form's delivery fields. First, this method loops through
	 * the fields in the "deliveryRequired" setting and check that each of the
	 * corresponding properties of this form are populated. Second, it loops
	 * through the fields in the "deliveryValidated" setting and ensures the
	 * field values are valid. (For example, that the email is in valid form.)
	 */
	public void validateDeliveryFields() {
		// Set up a Class for reflection
		Class classDefinition = this.getClass();
		Class[] argumentTypes = null;
		Object[] arguments = null;
		// Loop through each of the required delivery fields checking to see if
		// each is here
		String deliveryRequiredList = "";
		try {
			deliveryRequiredList = (String) getSettings().getValue(
					"deliveryRequired");
			if (log.isDebugEnabled())
				log.debug("delivery required: " + deliveryRequiredList);
		} catch (Exception e) {
		}
		if (deliveryRequiredList.length() > 0) {
			String[] deliveryRequiredArray = deliveryRequiredList.split(",");
			for (int i = 0; i < deliveryRequiredArray.length; i++) {
				String thisFieldString = deliveryRequiredArray[i];
				// Allow nulls for state field if 'others' are allowed.
				if (thisFieldString.equals("state")
						&& doesCountryAllowOtherStates(getDeliveryCountry())) {
					if (log.isDebugEnabled())
						log.debug("State not req. - country allows 'others': "
								+ getDeliveryCountry());
				} else {
					String thisMethodString = "getDelivery"
							+ formUtils().capitalize(thisFieldString);
					String thisValue = null;
					try {
						Method method = classDefinition.getMethod(
								thisMethodString, argumentTypes);
						thisValue = (String) method.invoke(this, arguments);
					} catch (Exception e) {
						log.error("Exception: " + e.toString(), e);
					}
					if (thisValue == null || thisValue.length() == 0) {
						// If not here, look up the label for the field and
						// create
						// an
						// error with it
						String deliveryPrefix = Resources.getMessage(
								getRequest(), "errors.deliveryPrefix");
						String key = "orderForm.delivery"
								+ formUtils().capitalize(thisFieldString);
						String message = getMessage("order", key, null);
						getErrors().add(
								ActionMessages.GLOBAL_MESSAGE,
								new ActionMessage("errors.required",
										deliveryPrefix + message));
					}
				}
			}
		}

		// Loop through each of the validated delivery fields and then validate
		// them
		String deliveryValidatedList = "";
		try {
			deliveryValidatedList = (String) getSettings().getValue(
					"deliveryValidated");
			if (log.isDebugEnabled())
				log.debug("Delivery validated: " + deliveryValidatedList);
		} catch (Exception e) {
		}
		if (deliveryValidatedList.length() > 0) {
			String[] deliveryValidatedArray = deliveryValidatedList.split(",");
			for (int i = 0; i < deliveryValidatedArray.length; i++) {
				String thisFieldString = deliveryValidatedArray[i];
				// if(log.isDebugEnabled()) log.debug("Validating field: " +
				// thisFieldString);
				String thisMethodString = "getDelivery"
						+ formUtils().capitalize(thisFieldString);
				String thisValue = null;
				try {
					Method method = classDefinition.getMethod(thisMethodString,
							argumentTypes);
					thisValue = (String) method.invoke(this, arguments);
				} catch (Exception e) {
					log.error("Exception: " + e.toString(), e);
				}
				// Validate only if the field was provided.
				if (thisValue != null && thisValue.length() > 0) {
					if (log.isDebugEnabled())
						log.debug("Validating field: " + thisFieldString
								+ " value: " + thisValue);
					// The state field is a little more complicated
					if (thisFieldString.equals("state")) {
						if (log.isDebugEnabled())
							log.debug("Validating the state field");
						validateDeliveryStateField();
					} else if (!isDeliveryFieldValid(thisFieldString, thisValue)) {
						String deliveryPrefix = Resources.getMessage(
								getRequest(), "errors.deliveryPrefix");
						String key = "orderForm.delivery"
								+ formUtils().capitalize(thisFieldString);
						String message = getMessage("order", key, null);
						getErrors().add(
								ActionMessages.GLOBAL_MESSAGE,
								new ActionMessage("errors.invalid",
										deliveryPrefix + message));
					}
				}
			}
		}
	}

	/**
	 * For a given field, validates the value of the field in this form. Emails
	 * are validated with a regular expression. Phone numbers are checked just
	 * to ensure at least 10 digits are present. And postal codes are checked
	 * for at least five digits if the country is the US.
	 * 
	 * @param thisFieldString
	 *            The name of the field to validate
	 * @param thisValue
	 *            The field's value
	 * @return True, if the field's value is valid
	 */
	public boolean isDeliveryFieldValid(String thisFieldString, String thisValue) {
		if (thisFieldString.equals("email1")
				|| thisFieldString.equals("email2")) {
			return formUtils().isValidEmail(thisValue);
		}
		if (thisFieldString.equals("phone1")
				|| thisFieldString.equals("phone2")) {
			return formUtils().isValidPhone(thisValue);
		}
		if (thisFieldString.equals("postalCode")) {
			return formUtils().isValidPostalCode(thisValue,
					getDeliveryCountry());
		}
		return false;
	}

	/**
	 * Validate this form's state field. One, if 'Other' was selected as the
	 * state, this method ensures that a value for the other field is present.
	 * Two, if 'Other' was not selected, the state is checked to make sure it
	 * matches with the submitted country.
	 */
	public void validateDeliveryStateField() {
		if (getDeliveryState().equals("Other") && getDeliveryOther() != null) {
			if (log.isDebugEnabled())
				log.debug("Populating state with value in other field.");
			setDeliveryState(getDeliveryOther());
		}
		// Make sure the submitted state belongs to the submitted country.
		if (log.isDebugEnabled())
			log.debug("Checking that the selected state and country match.");
		if (!stateAndCountryMatch(getDeliveryState(), getDeliveryCountry())) {
			String deliveryPrefix = Resources.getMessage(getRequest(),
					"errors.deliveryPrefix");
			getErrors().add(
					ActionMessages.GLOBAL_MESSAGE,
					new ActionMessage("errors.stateCountryMismatch",
							deliveryPrefix));
		}
	}

	/**
	 * Prepares the billingState, billingOther, deliveryState, and deliveryOther
	 * fields for presentation. Determines if a given <code>Customer</code>'s
	 * address uses a 'Other' state and if so populates the fields accordingly.
	 * 
	 * @param customer
	 *            The given customer whose address fields are being prepared.
	 */
	public void prepareStateAndOtherFields(Customer customer) {
		super.prepareStateAndOtherFields(customer);
		// Handle the Other and state fields for the delivery address
		if (customer.getFirstAddress() != null) {
			String ds = customer.getFirstAddress().getDeliveryState();
			if (ds != null && ds.length() > 0 && isOtherState(ds)) {
				setDeliveryState("Other");
				setDeliveryOther(customer.getFirstAddress().getDeliveryState());
			}
		}
	}

	/**
	 * Prepares the billingState, billingOther, deliveryState, and deliveryOther
	 * fields for presentation. Determines if a given <code>Order</code>'s
	 * address uses a 'Other' state and if so populates the fields accordingly.
	 * 
	 * @param order
	 *            The given order whose address fields are being prepared.
	 */
	public void prepareStateAndOtherFields(Order order) {
		super.prepareStateAndOtherFields(order);
		// Handle the Other and state fields for the delivery address
		if (order.firstDelivery() != null) {
			String ds = order.firstDelivery().getDeliveryState();
			if (ds != null && ds.length() > 0 && isOtherState(ds)) {
				setDeliveryState("Other");
				setDeliveryOther(order.firstDelivery().getDeliveryState());
			}
		}
	}
}
