package com.softslate.commerce.customer.order;

import java.lang.reflect.Method;
import java.util.Iterator;

import javax.servlet.http.HttpServletRequest;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.struts.action.ActionErrors;
import org.apache.struts.action.ActionMapping;
import org.apache.struts.action.ActionMessage;
import org.apache.struts.action.ActionMessages;
import org.apache.struts.validator.Resources;

import com.softslate.commerce.businessobjects.customer.Customer;
import com.softslate.commerce.businessobjects.order.Order;
import com.softslate.commerce.businessobjects.order.Country;
import com.softslate.commerce.businessobjects.order.State;
import com.softslate.commerce.businessobjects.order.StatesAndCountries;
import com.softslate.commerce.customer.core.BaseForm;

/**
 * Struts form class representing a user's billing address. The class is used
 * for submissions of billing address information in the customer account area.
 * It is also a superclass of
 * {@link com.softslate.commerce.customer.order.CheckoutAddressesForm}, which
 * is used for submissions to "/CheckoutAddresses.do", where both billing and
 * delivery addresses, as well as a shipping option, are submitted.
 * <p>
 * The <code>validate</code> method of this class loops through the
 * "billingRequired" database setting and makes sure each required field is
 * populated. It also checks to make sure the submitted state or province
 * corresponds to the submitted country, and that fields such as emails and
 * phone numbers are in a valid format.
 * 
 * @author David Tobey
 * 
 */
public class BillingAddressForm extends BaseForm {

	private static final long serialVersionUID = 3257008756612215601L;

	static Log log = LogFactory.getLog(BillingAddressForm.class);

	private String billingFirstName = null;

	private String billingLastName = null;

	private String billingOrganization = null;

	private String billingAddress1 = null;

	private String billingAddress2 = null;

	private String billingCity = null;

	private String billingState = null;

	private String billingOther = null;

	private String billingPostalCode = null;

	private String billingCountry = null;

	private String billingPhone1 = null;

	private String billingPhone2 = null;

	private String billingEmail1 = null;

	private String billingEmail2 = null;

	private String billingExtra1 = null;

	private String billingExtra2 = null;

	private String billingExtra3 = null;

	private String billingExtra4 = null;

	private String billingExtra5 = null;

	private String billingNotes = null;

	public String getBillingAddress1() {
		return billingAddress1;
	}

	public String getBillingAddress2() {
		return billingAddress2;
	}

	public String getBillingCity() {
		return billingCity;
	}

	public String getBillingCountry() {
		return billingCountry;
	}

	public String getBillingEmail1() {
		return billingEmail1;
	}

	public String getBillingEmail2() {
		return billingEmail2;
	}

	public String getBillingExtra1() {
		return billingExtra1;
	}

	public String getBillingExtra2() {
		return billingExtra2;
	}

	public String getBillingFirstName() {
		return billingFirstName;
	}

	public String getBillingLastName() {
		return billingLastName;
	}

	public String getBillingNotes() {
		return billingNotes;
	}

	public String getBillingOrganization() {
		return billingOrganization;
	}

	public String getBillingPhone1() {
		return billingPhone1;
	}

	public String getBillingPhone2() {
		return billingPhone2;
	}

	public String getBillingPostalCode() {
		return billingPostalCode;
	}

	public String getBillingState() {
		return billingState;
	}

	public String getBillingOther() {
		return billingOther;
	}

	public void setBillingAddress1(String string) {
		billingAddress1 = string;
	}

	public void setBillingAddress2(String string) {
		billingAddress2 = string;
	}

	public void setBillingCity(String string) {
		billingCity = string;
	}

	public void setBillingCountry(String string) {
		billingCountry = string;
	}

	public void setBillingEmail1(String string) {
		billingEmail1 = string;
	}

	public void setBillingEmail2(String string) {
		billingEmail2 = string;
	}

	public void setBillingExtra1(String string) {
		billingExtra1 = string;
	}

	public void setBillingExtra2(String string) {
		billingExtra2 = string;
	}

	public String getBillingExtra3() {
		return billingExtra3;
	}

	public void setBillingExtra3(String billingExtra3) {
		this.billingExtra3 = billingExtra3;
	}

	public String getBillingExtra4() {
		return billingExtra4;
	}

	public void setBillingExtra4(String billingExtra4) {
		this.billingExtra4 = billingExtra4;
	}

	public String getBillingExtra5() {
		return billingExtra5;
	}

	public void setBillingExtra5(String billingExtra5) {
		this.billingExtra5 = billingExtra5;
	}

	public void setBillingFirstName(String string) {
		billingFirstName = string;
	}

	public void setBillingLastName(String string) {
		billingLastName = string;
	}

	public void setBillingNotes(String string) {
		billingNotes = string;
	}

	public void setBillingOrganization(String string) {
		billingOrganization = string;
	}

	public void setBillingPhone1(String string) {
		billingPhone1 = string;
	}

	public void setBillingPhone2(String string) {
		billingPhone2 = string;
	}

	public void setBillingPostalCode(String string) {
		billingPostalCode = string;
	}

	public void setBillingState(String string) {
		billingState = string;
	}

	public void setBillingOther(String string) {
		billingOther = string;
	}

	private String csrfToken = null;

	public String getCsrfToken() {
		return csrfToken;
	}

	public void setCsrfToken(String csrfToken) {
		this.csrfToken = csrfToken;
	}

	public ActionErrors validate(ActionMapping mapping,
			HttpServletRequest request) {
		if (log.isInfoEnabled())
			log.info("Starting validation.");
		initializeProperties(mapping, request);

		validateBillingFields();
		
		// Ensure that the form submission is coming from the site itself
		if (getCsrfToken() == null || getCsrfToken().length() == 0
				|| !getCsrfToken().equals(getRequest().getSession().getAttribute("csrfToken"))) {
			getErrors().add(ActionMessages.GLOBAL_MESSAGE, new ActionMessage("errors.csrfError"));
		}

		if (log.isDebugEnabled())
			log.debug("Finished validation.");
		if (log.isInfoEnabled() && !getErrors().isEmpty())
			log.info("Validation error(s): " + getErrors().toString());
		return getErrors();
	}

	/**
	 * Validates the billing fields of this form. First, this method loops
	 * through the fields in the "billingRequired" setting and check that each
	 * of the corresponding properties of this form are populated. Second, it
	 * loops through the fields in the "billingValidated" setting and ensures
	 * the field values are valid. (For example, that the email is in valid
	 * form.)
	 */
	public void validateBillingFields() {
		// Set up a Class for reflection
		Class classDefinition = this.getClass();
		Class[] argumentTypes = null;
		Object[] arguments = null;
		// Loop through each of the required billing fields checking to see if
		// each is here
		String billingRequiredList = "";
		try {
			billingRequiredList = (String) getSettings().getValue(
					"billingRequired");
			if (log.isDebugEnabled())
				log.debug("Billing required: " + billingRequiredList);
		} catch (Exception e) {
		}
		if (billingRequiredList.length() > 0) {
			String[] billingRequiredArray = billingRequiredList.split(",");
			for (int i = 0; i < billingRequiredArray.length; i++) {
				String thisFieldString = billingRequiredArray[i];
				// Allow nulls for state field if 'others' are allowed.
				if (thisFieldString.equals("state")
						&& doesCountryAllowOtherStates(getBillingCountry())) {
					if (log.isDebugEnabled())
						log.debug("State not req. - country allows 'others': "
								+ getBillingCountry());
				} else {
					// if(log.isDebugEnabled()) log.debug("Validating field: " +
					// thisFieldString);
					String thisMethodString = "getBilling"
							+ formUtils().capitalize(thisFieldString);
					String thisValue = null;
					try {
						Method method = classDefinition.getMethod(
								thisMethodString, argumentTypes);
						thisValue = (String) method.invoke(this, arguments);
					} catch (Exception e) {
						if (log.isDebugEnabled())
							log.debug("Exception: " + e.toString());
					}
					if (thisValue == null || thisValue.length() == 0) {
						// If not here, look up the label for the field and
						// create
						// an error with it
						String billingPrefix = Resources.getMessage(
								getRequest(), "errors.billingPrefix");
						String key = "orderForm.billing"
								+ formUtils().capitalize(thisFieldString);
						String message = getMessage("order", key, null);
						getErrors().add(
								ActionMessages.GLOBAL_MESSAGE,
								new ActionMessage("errors.required",
										billingPrefix + message));
					}
				}
			}
		}

		// Loop through each of the validated billing fields and then validate
		// them
		String billingValidatedList = "";
		try {
			billingValidatedList = (String) getSettings().getValue(
					"billingValidated");
			if (log.isDebugEnabled())
				log.debug("Billing validated: " + billingValidatedList);
		} catch (Exception e) {
		}
		if (billingValidatedList.length() > 0) {
			String[] billingValidatedArray = billingValidatedList.split(",");
			for (int i = 0; i < billingValidatedArray.length; i++) {
				String thisFieldString = billingValidatedArray[i];
				// if(log.isDebugEnabled()) log.debug("Validating field: " +
				// thisFieldString);
				String thisMethodString = "getBilling"
						+ formUtils().capitalize(thisFieldString);
				String thisValue = null;
				try {
					Method method = classDefinition.getMethod(thisMethodString,
							argumentTypes);
					thisValue = (String) method.invoke(this, arguments);
				} catch (Exception e) {
					if (log.isDebugEnabled())
						log.debug("Exception: " + e.toString());
				}
				// Validate only if the field was provided.
				if (thisValue != null && thisValue.length() > 0) {
					if (log.isDebugEnabled())
						log.debug("Validating field: " + thisFieldString
								+ " value: " + thisValue);
					// The state field is a little more complicated
					if (thisFieldString.equals("state")) {
						if (log.isDebugEnabled())
							log.debug("Validating the state field");
						validateBillingStateField();
					} else if (!isBillingFieldValid(thisFieldString, thisValue)) {
						String billingPrefix = Resources.getMessage(
								getRequest(), "errors.billingPrefix");
						String key = "orderForm.billing"
								+ formUtils().capitalize(thisFieldString);
						String message = getMessage("order", key, null);
						getErrors().add(
								ActionMessages.GLOBAL_MESSAGE,
								new ActionMessage("errors.invalid",
										billingPrefix + message));
					}
				}
			}
		}
	}

	/**
	 * For a given field, validates the value of the field in this form. Emails
	 * are validated with a regular expression. Phone numbers are checked just
	 * to ensure at least 10 digits are present. And postal codes are checked
	 * for at least five digits if the country is the US.
	 * 
	 * @param thisFieldString
	 *            The name of the field to validate
	 * @param thisValue
	 *            The field's value
	 * @return True, if the field's value is valid
	 */
	public boolean isBillingFieldValid(String thisFieldString, String thisValue) {
		if (thisFieldString.equals("email1")
				|| thisFieldString.equals("email2")) {
			return formUtils().isValidEmail(thisValue);
		}
		if (thisFieldString.equals("phone1")
				|| thisFieldString.equals("phone2")) {
			return formUtils().isValidPhone(thisValue);
		}
		if (thisFieldString.equals("postalCode")) {
			return formUtils()
					.isValidPostalCode(thisValue, getBillingCountry());
		}
		return false;
	}

	/**
	 * Validate this form's state field. One, if 'Other' was selected as the
	 * state, this method ensures that a value for the other field is present.
	 * Two, if 'Other' was not selected, the state is checked to make sure it
	 * matches with the submitted country.
	 */
	public void validateBillingStateField() {
		if (getBillingState().equals("Other") && getBillingOther() != null) {
			if (log.isDebugEnabled())
				log.debug("Populating state with value in other field.");
			setBillingState(getBillingOther());
		}
		// Make sure the submitted state belongs to the submitted country.
		if (log.isDebugEnabled())
			log.debug("Checking that the selected state and country match.");
		if (!stateAndCountryMatch(getBillingState(), getBillingCountry())) {
			String billingPrefix = Resources.getMessage(getRequest(),
					"errors.billingPrefix");
			getErrors().add(
					ActionMessages.GLOBAL_MESSAGE,
					new ActionMessage("errors.stateCountryMismatch",
							billingPrefix));
		}
	}

	/**
	 * Checks to make sure this form's state and country match. In other words,
	 * if the given state is contained in the <code>sscState</code> table, the
	 * given country must correspond to that state's country. If the given state
	 * is a 'Other' state, the given country must have its allowOtherStates flag
	 * set.
	 * 
	 * @param state
	 *            The string representing the state to be checked. This will be
	 *            the two-letter state code from the <code>code</code> field
	 *            of the <code>sscState</code> table, or some other string
	 *            inputted by the user (if it is a 'Other' state).
	 * @param country
	 *            The string representing the country to be checked. This will
	 *            be the two-letter country code from the <code>code</code>
	 *            field of the <code>sscCountry</code> table.
	 * @return true, if the state and country match.
	 */
	public boolean stateAndCountryMatch(String state, String country) {
		// Make sure the submitted state belongs to the submitted country.
		if (log.isDebugEnabled())
			log.debug("Checking that the selected state and country match.");
		StatesAndCountries statesAndCountries = (StatesAndCountries) getServlet()
				.getServletContext().getAttribute("statesAndCountries");
		Iterator iteratorStates = statesAndCountries.getStates().iterator();
		Iterator iteratorCountries = statesAndCountries.getCountries()
				.iterator();
		boolean allowOtherStates = false;
		int countryID = 0;
		int countryID2 = 0;
		boolean stateFound = false;
		boolean countryFound = false;
		while (iteratorCountries.hasNext() && !countryFound) {
			Country thisCountry = (Country) iteratorCountries.next();
			String thisCountryCode = thisCountry.getCode();
			if (thisCountryCode.equals(country)) {
				countryID = thisCountry.getCountryID();
				allowOtherStates = thisCountry.getAllowOtherStates();
				countryFound = true;
			}
		}
		if (log.isDebugEnabled())
			log.debug("Country ID associated with country: " + countryID);
		if (allowOtherStates) {
			return true;
		}
		while (iteratorStates.hasNext() && !stateFound) {
			State thisState = (State) iteratorStates.next();
			String thisStateCode = (String) thisState.getCode();
			if (thisStateCode.equals(state)) {
				countryID2 = thisState.getCountryID();
				stateFound = true;
			}
		}
		if (log.isDebugEnabled())
			log.debug("Country ID associated with state: " + countryID2);
		if (countryID != 0 && countryID != countryID2) {
			return false;
		}
		return true;
	}

	/**
	 * Prepares the billingState and billingOther fields for presentation in a
	 * form. Determines if a given <code>Customer</code>'s billing address
	 * uses a 'Other' state and if so populates the billingOther and
	 * billingState fields accordingly. If it does use a 'Other' state, this
	 * method populates the billingState field with the string "Other" and
	 * populates the billingOther field with the value of billingState, so the
	 * address forms will accurately represent the customer's state.
	 * 
	 * @param customer
	 *            The given customer whose address fields are being prepared.
	 */
	public void prepareStateAndOtherFields(Customer customer) {
		// Handle the Other and state fields for the billing address
		String bs = customer.getBillingState();
		if (bs != null && bs.length() > 0 && isOtherState(bs)) {
			setBillingState("Other");
			setBillingOther(customer.getBillingState());
		}
	}

	/**
	 * Prepares the billingState and billingOther fields for presentation in a
	 * form. Determines if a given <code>Order</code>'s billing address
	 * uses a 'Other' state and if so populates the billingOther and
	 * billingState fields accordingly. If it does use a 'Other' state, this
	 * method populates the billingState field with the string "Other" and
	 * populates the billingOther field with the value of billingState, so the
	 * address forms will accurately represent the customer's state.
	 * 
	 * @param order
	 *            The given order whose address fields are being prepared.
	 */
	public void prepareStateAndOtherFields(Order order) {
		// Handle the Other and state fields for the billing address
		String bs = order.getBillingState();
		if (bs != null && bs.length() > 0 && isOtherState(bs)) {
			setBillingState("Other");
			setBillingOther(order.getBillingState());
		}
	}

	/**
	 * Determines if a given state is a 'Other' state. In other words,
	 * determines if the value of the state is contained in the
	 * <code>sscState</code> database table or if it corresponds to a state or
	 * province in a foreign country not supported by <code>sscState</code>.
	 * 
	 * @param state
	 *            The string representing the state to be checked. This will be
	 *            the two-letter state code from the <code>code</code> field
	 *            of the <code>sscState</code> table, or some other string
	 *            inputted by the user (if it is a 'Other' state).
	 * @return true, if the given state is a 'Other' state.
	 */
	public boolean isOtherState(String state) {
		StatesAndCountries statesAndCountries = (StatesAndCountries) getServlet()
				.getServletContext().getAttribute("statesAndCountries");
		Iterator iteratorStates = statesAndCountries.getStates().iterator();
		while (iteratorStates.hasNext()) {
			State thisState = (State) iteratorStates.next();
			String thisStateCode = (String) thisState.getCode();
			if (thisStateCode.equals(state)) {
				return false;
			}
		}
		return true;
	}

	/**
	 * Determines if a given country is set to allow 'Other' states.
	 * 
	 * @param country
	 *            The string representing the country to be checked. This will
	 *            be the two-letter country code from the <code>code</code>
	 *            field of the <code>sscCountry</code> table.
	 * @return true, if the given country is set to allow 'Other' states.
	 */
	public boolean doesCountryAllowOtherStates(String country) {
		StatesAndCountries statesAndCountries = (StatesAndCountries) getServlet()
				.getServletContext().getAttribute("statesAndCountries");
		Iterator iteratorCountries = statesAndCountries.getCountries()
				.iterator();
		while (iteratorCountries.hasNext()) {
			Country thisCountry = (Country) iteratorCountries.next();
			String thisCountryCode = (String) thisCountry.getCode();
			if (thisCountryCode.equals(country)) {
				return thisCountry.getAllowOtherStates();
			}
		}
		return false;
	}

}
