package com.softslate.commerce.customer.order;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.struts.action.ActionForm;
import org.apache.struts.action.ActionForward;
import org.apache.struts.action.ActionMapping;
import org.apache.struts.action.ActionMessage;
import org.apache.struts.action.ActionMessages;

import com.softslate.commerce.customer.core.ActionUtils;
import com.softslate.commerce.customer.core.BaseForm;

/**
 * Struts action class for submissions of the checkout shipping form. Used for
 * requests to "/CheckoutShipping.do".
 * <p>
 * The <code>execute</code> method of this class first clears the session's
 * "shippingAndTaxesProcessed" attribute. It then runs
 * {@link com.softslate.commerce.customer.core.ActionUtils#findCheckoutScreen(BaseForm)}
 * to verifiy that the user has made it this far in the checkout process.
 * Finally, it invokes
 * {@link com.softslate.commerce.customer.order.CheckoutAction#processCheckoutShipping(BaseForm)}
 * to process the shipping and taxes.
 * 
 * @author David Tobey
 */
public class CheckoutShippingAction extends CheckoutAction {
	static Log log = LogFactory.getLog(CheckoutShippingAction.class);

	public ActionForward execute(ActionMapping mapping, ActionForm form,
			HttpServletRequest request, HttpServletResponse response)
			throws Exception {
		if (log.isInfoEnabled())
			log.info("Starting action.");

		BaseForm baseForm = (BaseForm) form;
		baseForm.setServlet(getServlet());
		baseForm.initializeProperties(mapping, request);

		synchronized (baseForm.getRequest().getSession()) {
			// Make sure the user is allowed to have this action performed.
			baseForm.getRequest().getSession().setAttribute(
					"shippingAndTaxesProcessed", "false");
			ActionUtils actionUtils = (ActionUtils) baseForm
					.getBusinessObjectFactory().createUtilsObject(
							"actionUtilsImplementer");
			String checkoutScreen = actionUtils.findCheckoutScreen(baseForm);
			if (!(checkoutScreen.equals("checkoutShipping") || checkoutScreen.equals("checkoutAjaxOnePage"))) {
				if (request.getAttribute("isApiRequest") != null && (Boolean) request.getAttribute("isApiRequest")) {
					baseForm.getErrors().add(ActionMessages.GLOBAL_MESSAGE,
							new ActionMessage("errors.checkoutShippingWrongStep"));
					saveErrors(baseForm.getRequest(), (ActionMessages) baseForm.getErrors());
				}
				return baseForm.getMapping().findForward(checkoutScreen);
			}
			// Process the form.
			return processCheckoutShipping(baseForm);
		}
	}
}
