package com.softslate.commerce.daos.core;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collection;
import java.util.HashMap;
import java.util.Iterator;
import java.util.LinkedHashSet;
import java.util.Map;

import org.apache.commons.beanutils.BeanUtils;
import org.apache.commons.beanutils.PropertyUtils;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.hibernate.Criteria;
import org.hibernate.HibernateException;
import org.hibernate.Session;
import org.hibernate.criterion.Junction;
import org.hibernate.criterion.MatchMode;
import org.hibernate.criterion.Order;
import org.hibernate.criterion.Projections;
import org.hibernate.criterion.Restrictions;

import com.softslate.commerce.businessobjects.core.ApplicationMessage;
import com.softslate.commerce.businessobjects.core.BusinessObjectFactory;

/**
 * @author David Tobey
 */
public class BaseAdminGatewayDAOHibernate extends BaseDAO implements
		DAOInterface {

	static Log log = LogFactory.getLog(BaseAdminGatewayDAOHibernate.class);

	public Collection loadItemsFromIDs(String[] itemIDs, String table,
			String module) throws Exception {
		return null;
	}

	public Map editDeleteItems(Map parameters, BusinessObjectFactory bof)
			throws Exception {
		Map result = new HashMap();
		Collection<ApplicationMessage> applicationMessages = new ArrayList();
		String idField = (String) parameters.get("idField");
		String[] idValues = (String[]) parameters.get("idValues");
		Integer[] idIntegers = (Integer[]) parameters.get("idIntegers");
		String parentIDField = (String) parameters.get("parentIDField");
		String parentID = (String) parameters.get("parentID");
		String manyToManyClass = (String) parameters.get("manyToManyClass");
		Class entity = (Class) parameters.get("entity");
		String[] deleteID = (String[]) parameters.get("deleteID");
		String[] assigned = (String[]) parameters.get("assigned");
		// Debugging
		if (log.isDebugEnabled())
			log.debug("assigned: " + assigned);
		if (log.isDebugEnabled())
			log.debug("parentIDField: " + parentIDField);
		if (log.isDebugEnabled())
			log.debug("parentID: " + parentID);
		if (log.isDebugEnabled())
			log.debug("idField: " + idField);
		if (log.isDebugEnabled())
			log.debug("idValues: " + idValues);
		if (log.isDebugEnabled())
			log.debug("manyToManyClass: " + manyToManyClass);
		Collection items = null;
		Session session = getDaoFactory().startSession();
		try {

			// Build the query's criteria using the Criteria API
			Criteria crit = session.createCriteria(entity);
			crit.add(Restrictions.in(idField, idIntegers));
			items = new LinkedHashSet(crit.list());
			Iterator iterator = items.iterator();
			while (iterator.hasNext()) {
				Object item = (Object) iterator.next();
				String idValue = BeanUtils.getProperty(item, idField);
				if (deleteID != null
						&& Arrays.asList(deleteID).contains(idValue)) {

					// Delete the item
					if (log.isDebugEnabled())
						log.debug("Deleting item with id: " + idValue);
					session.delete(item);
					session.flush();
				} else {
					for (int i = 0; i < idValues.length; i++) {
						if (idValue.equals(idValues[i])) {

							// Set each of the properties that was displayed
							Map incomingProperties = incomingPropertiesByIndex(parameters, i);
							Map originalParameters = PropertyUtils.describe(item);

							// Call triggers for changed items
							Map results = postChangingItemEvent(item, incomingProperties, bof);
							if (results != null && results.get("applicationMessages") != null) {
								applicationMessages.addAll((Collection<ApplicationMessage>) results
										.get("applicationMessages"));
							}
							BeanUtils.copyProperties(item, incomingProperties);

							// Update the item
							if (log.isDebugEnabled())
								log.debug("Updating item with id: " + idValue);
							session.update(item);

							// Add and delete assignments
							editDeleteAssignments(parameters, item, bof, i);
							session.flush();

							results = postChangedItemEvent(item, originalParameters, bof);
							if (results != null && results.get("applicationMessages") != null) {
								applicationMessages.addAll((Collection<ApplicationMessage>) results
										.get("applicationMessages"));
							}
						}
					}
				}
			}
			getDaoFactory().commitTransaction();
			getDaoFactory().getSession().clear();
			result.put("applicationMessages", applicationMessages);
		} catch (HibernateException he) {
			if (log.isErrorEnabled())
				log.error("Hibernate error: " + he.toString());
			throw new DataAccessException(he);
		}
		return result;
	}

	public void setPropertiesDynamically(Map parameters, Object item, int index)
			throws Exception {
		String[] displayFields = (String[]) parameters.get("displayFields");
		for (int j = 0; j < displayFields.length; j++) {
			String displayField = displayFields[j];
			Object[] fieldValues = (Object[]) parameters.get(displayField);
			if (fieldValues != null) {
				BeanUtils.copyProperty(item, displayField,
						fieldValues[index]);
			}
		}
	}

	public Map incomingPropertiesByIndex(Map parameters, int index) throws Exception {
		Map incomingProperties = new HashMap();
		String[] displayFields = (String[]) parameters.get("displayFields");
		for (int j = 0; j < displayFields.length; j++) {
			String displayField = displayFields[j];
			Object[] fieldValues = (Object[]) parameters.get(displayField);
			if (fieldValues != null) {
				incomingProperties.put(displayField, fieldValues[index]);
			}
		}
		return incomingProperties;
	}

	public Map postChangingItemEvent(Object item, Map parameters, BusinessObjectFactory bof) throws Exception {
		return null;
	}

	public Map postChangedItemEvent(Object item, Map parameters, BusinessObjectFactory bof) throws Exception {
		return null;
	}

	public Map editDeleteAssignments(Map parameters, Object item,
			BusinessObjectFactory bof, int index) throws Exception {
		Map result = null;
		String idField = (String) parameters.get("idField");
		Integer[] idIntegers = (Integer[]) parameters.get("idIntegers");
		String parentIDField = (String) parameters.get("parentIDField");
		String parentID = (String) parameters.get("parentID");
		String parentKey = (String) parameters.get("parentKey");
		String manyToManyKey = (String) parameters.get("manyToManyKey");
		String manyToManyClass = (String) parameters.get("manyToManyClass");
		String entityProperty = (String) parameters.get("entityProperty");
		String parentProperty = (String) parameters.get("parentProperty");
		String[] assigned = (String[]) parameters.get("assigned");
		Session session = getDaoFactory().getSession();

		// Add and delete assignments
		if (assigned != null && assigned.length > index) {
			Class m2m = Class.forName(manyToManyClass);
			Criteria m2mCrit = session.createCriteria(m2m);
			m2mCrit.add(Restrictions.eq(idField, idIntegers[index]));
			m2mCrit.add(Restrictions.eq(parentIDField, new Integer(Integer
					.parseInt(parentID))));
			if (manyToManyKey != null && manyToManyKey.equals("relatedProductImplementer")
					&& parameters.containsKey("relatedTypeFilter")) {
				String relatedType = (String) parameters.get("relatedTypeFilter");
				if (relatedType == null) {
					m2mCrit.add(Restrictions.isNull("relatedType"));
				}
				else {
					m2mCrit.add(Restrictions.eq("relatedType", relatedType));
				}
			}
			Object o = m2mCrit.uniqueResult();
			boolean deleteAssignment = o != null
					&& (assigned[index].equals("0") || assigned[index]
							.equals("false"));
			boolean insertAssignmeent = o == null
					&& (assigned[index].equals("1") || assigned[index]
							.equals("true"));
			boolean updateAssignment = o != null && !deleteAssignment;
			if (log.isDebugEnabled())
				log.debug("Delete/Insert/Update assignment: "
						+ deleteAssignment + "/" + insertAssignmeent + "/"
						+ updateAssignment);

			if (deleteAssignment) {

				// Delete the assignment
				session.delete(o);
			} else if (insertAssignmeent || updateAssignment) {

				// Insert new or update existing
				if (insertAssignmeent) {
					o = bof.createObject(manyToManyKey);
				}
				Object p = bof.createObject(parentKey);
				BeanUtils.copyProperty(p, parentIDField, parentID);
				if (parentIDField.equals("parentProductID")) {
					BeanUtils.copyProperty(p, "productID", parentID);
				}
				setPropertiesDynamically(parameters, o, index);
				BeanUtils.copyProperty(o, parentProperty, p);
				BeanUtils.copyProperty(o, entityProperty, item);
				session.saveOrUpdate(o);
			}
		}
		return result;
	}

	public Map loadItemsAndCount(Map parameters) throws Exception {
		// Define the parentID field if necessary
		String idField = (String) parameters.get("idField");
		String assignments = (String) parameters.get("assignments");
		if (log.isDebugEnabled())
			log.debug("idField: " + idField);
		Map itemsAndCount = new HashMap();
		Collection items = null;
		int count = 0;
		if (assignments != null && assignments.equals("all")) {
			// Load all items, then load assigned items into 'assignedItems'
			items = loadItems(parameters);
			count = loadItemsCount(parameters);
			itemsAndCount.put("items", items);
			itemsAndCount.put("itemCount", new Integer(count));
			parameters.put("assignments", "assigned");
			parameters.put("firstRow", null);
			parameters.put("itemsPerPage", null);
			Collection assignedItems = loadItems(parameters);
			assignedItems = getDAOUtils().getCollectionFromBeans(assignedItems,
					idField);
			itemsAndCount.put("assignedItems", assignedItems);
			if (log.isDebugEnabled())
				log.debug("assignedItems: " + assignedItems);
		} else if (assignments != null && assignments.equals("assigned")) {
			// Load assigned items, label it both 'assignedItems' and
			// 'items'
			items = loadItems(parameters);
			count = loadItemsCount(parameters);
			itemsAndCount.put("items", items);
			itemsAndCount.put("itemCount", new Integer(count));
			Collection assignedItems = getDAOUtils().getCollectionFromBeans(
					items, idField);
			itemsAndCount.put("assignedItems", assignedItems);
		} else if (assignments != null && assignments.equals("unassigned")) {
			// Load assigned items, then use those to load unassigned items.
			String firstRow = (String) parameters.get("firstRow");
			String itemsPerPage = (String) parameters.get("itemsPerPage");
			parameters.put("assignments", "assigned");
			parameters.put("firstRow", null);
			parameters.put("itemsPerPage", null);
			Collection assignedItems = loadItems(parameters);
			Collection assignedItemsList = getDAOUtils()
					.getCollectionFromBeans(assignedItems, idField);
			parameters.put("assignments", "unassigned");
			parameters.put("firstRow", firstRow);
			parameters.put("itemsPerPage", itemsPerPage);
			parameters.put("assignedItemsList", assignedItemsList);
			items = loadItems(parameters);
			count = loadItemsCount(parameters);
			itemsAndCount.put("items", items);
			itemsAndCount.put("itemCount", new Integer(count));
			itemsAndCount.put("assignedItems", assignedItems);
		} else {
			// Just load the items
			items = loadItems(parameters);
			count = loadItemsCount(parameters);
			itemsAndCount.put("items", items);
			itemsAndCount.put("itemCount", new Integer(count));
		}
		return itemsAndCount;
	}

	public Collection loadItems(Map parameters) throws DataAccessException {
		String parentID = (String) parameters.get("parentID");
		Class entity = (Class) parameters.get("entity");
		String assignments = (String) parameters.get("assignments");
		Collection assignedItemsList = (Collection) parameters
				.get("assignedItemsList");
		String searchString = (String) parameters.get("searchString");
		String sortByString = (String) parameters.get("sortByString");
		String sortAscOrDesc = (String) parameters.get("sortAscOrDesc");
		String firstRow = (String) parameters.get("firstRow");
		String itemsPerPage = (String) parameters.get("itemsPerPage");
		// Debugging
		if (log.isDebugEnabled())
			log.debug("assignments: " + assignments);
		if (log.isDebugEnabled())
			log.debug("assignedItemsList: " + assignedItemsList);
		if (log.isDebugEnabled())
			log.debug("parentID: " + parentID);
		if (log.isDebugEnabled())
			log.debug("Search string: " + searchString);
		if (log.isDebugEnabled())
			log.debug("sortByString: "
					+ (String) parameters.get("sortByString"));
		if (log.isDebugEnabled())
			log.debug("sortAscOrDesc: "
					+ (String) parameters.get("sortAscOrDesc"));
		if (log.isDebugEnabled())
			log.debug("firstRow (+1): " + firstRow);
		if (log.isDebugEnabled())
			log.debug("itemsPerPage: " + itemsPerPage);

		Collection results = null;
		Session session = getDaoFactory().startSession();
		try {
			// Build the query's criteria using the Criteria API
			Criteria crit = session.createCriteria(entity);
			crit = processCommonCriteria(crit, parameters);
			// If there is a sortByString, set the order by clause
			if (sortByString != null) {
				String[] sortBys = sortByString.split(",");
				for (int i = 0; i < sortBys.length; i++) {
					if (sortAscOrDesc != null && sortAscOrDesc.equals("DESC")) {
						crit.addOrder(Order.desc(sortBys[i]));
					} else {
						crit.addOrder(Order.asc(sortBys[i]));
					}
				}
			}
			// Set the paging paramaters
			if (firstRow != null && itemsPerPage != null) {
				crit.setFirstResult(Integer.parseInt(firstRow) - 1);
				crit.setMaxResults(Integer.parseInt(itemsPerPage));
			}
			// Run the query
			results = new LinkedHashSet(crit.list());
		} catch (HibernateException he) {
			if (log.isErrorEnabled())
				log.error("Hibernate error: ", he);
			throw new DataAccessException(he);
		}
		return results;
	}

	public int loadItemsCount(Map parameters) throws DataAccessException {
		String idField = (String) parameters.get("idField");
		String parentID = (String) parameters.get("parentID");
		Class entity = (Class) parameters.get("entity");
		String assignments = (String) parameters.get("assignments");
		Collection assignedItemsList = (Collection) parameters
				.get("assignedItemsList");
		String searchString = (String) parameters.get("searchString");
		// Debugging
		if (log.isDebugEnabled())
			log.debug("assignments: " + assignments);
		if (log.isDebugEnabled())
			log.debug("assignedItemsList: " + assignedItemsList);
		if (log.isDebugEnabled())
			log.debug("parentID: " + parentID);
		if (log.isDebugEnabled())
			log.debug("Search string: " + searchString);

		Long result = null;
		Session session = getDaoFactory().startSession();
		try {
			// Build the query's criteria using the Criteria API
			Criteria crit = session.createCriteria(entity);
			crit = processCommonCriteria(crit, parameters);
			crit.setProjection(Projections.countDistinct(idField));
			result = (Long) crit.uniqueResult();
		} catch (HibernateException he) {
			if (log.isErrorEnabled())
				log.error("Hibernate error: " + he.toString());
			throw new DataAccessException(he);
		}
		return result.intValue();
	}

	public Criteria processCommonCriteria(Criteria crit, Map parameters) {
		String parentID = (String) parameters.get("parentID");
		String parentIDField = (String) parameters.get("parentIDField");
		String idField = (String) parameters.get("idField");
		String manyToManyProp = (String) parameters.get("manyToManyProp");
		String parentProperty = (String) parameters.get("parentProperty");
		String joinManyToManyParent = (String) parameters
				.get("joinManyToManyParent");
		String assignments = (String) parameters.get("assignments");
		Collection assignedItemsList = (Collection) parameters
				.get("assignedItemsList");
		String searchString = (String) parameters.get("searchString");
		String[] searchableFields = (String[]) parameters
				.get("searchableFields");
		String[] searchableIntFields = (String[]) parameters
				.get("searchableIntFields");
		if (assignments != null && assignments.equals("assigned")) {
			// If querying for assigned items, join with the many-to-many
			// linking table, and optionally, that table's parent
			Criteria joinCrit = crit.createCriteria(manyToManyProp);
			if (joinManyToManyParent != null
					&& joinManyToManyParent.equals("true")) {
				Criteria parentJoinCrit = joinCrit
						.createCriteria(parentProperty);
				parentJoinCrit.add(Restrictions.eq(parentIDField, new Integer(
						Integer.parseInt(parentID))));
			} else {
				joinCrit.add(Restrictions.eq(parentIDField, new Integer(Integer
						.parseInt(parentID))));
			}
		} else if (assignedItemsList != null && assignments != null
				&& assignments.equals("unassigned")
				&& !assignedItemsList.isEmpty()) {
			// If querying for unassigned items, restrict assigned ids from
			// the results
			crit.add(Restrictions.not(Restrictions.in(idField,
					assignedItemsList)));
		} else if ((assignments == null || assignments.length() == 0)
				&& parentID != null && parentID.length() > 0) {
			// If this is a query for child items, restrict the results by
			// the parent id.
			if (parentIDField.startsWith("parent")) {
				String actualParentField = parentIDField.substring(6, 7).toLowerCase() + parentIDField.substring(7);
				crit.add(Restrictions.eq(actualParentField, new Integer(Integer
						.parseInt(parentID))));
			} else {
				crit.add(Restrictions.eq(parentIDField, new Integer(Integer
						.parseInt(parentID))));
			}
		}
		// Set the filter criteria
		if (searchString != null && searchString.length() > 0) {
			Junction search = Restrictions.disjunction();
			// Test for a match on integer fields
			if (searchableIntFields != null) {
				try {
					Integer searchInteger = new Integer(Integer
							.parseInt(searchString));
					for (int i = 0; i < searchableIntFields.length; i++) {
						search.add(Restrictions.eq(searchableIntFields[i],
								searchInteger));
					}
				} catch (NumberFormatException nfe) {
					// Swallow
				}
			}
			// Test for a match on string fields
			if (searchableFields != null) {
				searchString = getDAOUtils().prepareSearchString(searchString);
				for (int i = 0; i < searchableFields.length; i++) {
					if (searchableFields[i] != null && searchableFields[i].length() > 0) {
						search.add(Restrictions.ilike(searchableFields[i], searchString, MatchMode.EXACT));
					}
				}
			}
			crit.add(search);
		}
		return crit;
	}
}
