package com.softslate.commerce.daos.customer;

import org.apache.commons.beanutils.PropertyUtils;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.hibernate.HibernateException;
import org.hibernate.Session;

import com.softslate.commerce.businessobjects.customer.CustomerWishList;
import com.softslate.commerce.daos.core.BaseDAO;
import com.softslate.commerce.daos.core.DataAccessException;

/**
 * Generic DAO to manage the <code>sscCustomerWishList</code> table of the
 * database.
 * 
 * @author Jason McWilliams
 */
public class CustomerWishListDAOHibernate extends BaseDAO implements
        CustomerWishListDAO {

    static Log log = LogFactory.getLog(CustomerWishListDAOHibernate.class);

    private CustomerWishList customerWishList = null;

    public CustomerWishList getCustomerWishList() {
        return (this.customerWishList);
    }

    public void setCustomerWishList(CustomerWishList customerWishList) {
        this.customerWishList = customerWishList;
    }

    public void loadCustomerWishListFromID() throws Exception {
        Session session = getDaoFactory().startSession();
        try {
            CustomerWishList record = (CustomerWishList) session.get(
                    CustomerWishList.class, new Integer(getCustomerWishList()
                            .getCustomerWishListID()));
            PropertyUtils.copyProperties(getCustomerWishList(), record);
            setCustomerWishList(record);
        } catch (HibernateException he) {
            if (log.isErrorEnabled())
                log.error("Hibernate error: " + he.toString());
            throw new DataAccessException(he);
        }
        return;
    }

    public void insertCustomerWishList() throws DataAccessException {
        insertCustomerWishList(false);
    }

    public void insertCustomerWishList(boolean commit)
            throws DataAccessException {
        Session session = getDaoFactory().startSession();
        try {
        	getCustomerWishList().setCreated(formatDateTime(new java.util.Date()));
			getCustomerWishList().setLastUpdated(formatDateTime(new java.util.Date()));
            session.save(getCustomerWishList());
            if (commit) {
                getDaoFactory().commitTransaction();
            }
        } catch (HibernateException he) {
            if (log.isErrorEnabled())
                log.error("Hibernate error: " + he.toString());
            throw new DataAccessException(he);
        }
        return;
    }

    public void updateCustomerWishList() throws DataAccessException {
        updateCustomerWishList(false);
    }

    public void updateCustomerWishList(boolean commit)
            throws DataAccessException {
        Session session = getDaoFactory().startSession();
        try {
        	getCustomerWishList().setLastUpdated(formatDateTime(new java.util.Date()));
            session.update(getCustomerWishList());
            if (commit) {
                getDaoFactory().commitTransaction();
            }
        } catch (HibernateException he) {
            if (log.isErrorEnabled())
                log.error("Hibernate error: " + he.toString());
            throw new DataAccessException(he);
        }
        return;
    }

    public void deleteCustomerWishList() throws Exception {
        deleteCustomerWishList(false);
    }

    public void deleteCustomerWishList(boolean commit) throws Exception {
        Session session = getDaoFactory().startSession();
        try {
            setCustomerWishList((CustomerWishList) session.load(
                    CustomerWishList.class, new Integer(getCustomerWishList()
                            .getCustomerWishListID())));
            session.delete(getCustomerWishList());
            if (commit) {
                getDaoFactory().commitTransaction();
            }
        } catch (HibernateException he) {
            if (log.isErrorEnabled())
                log.error("Hibernate error: " + he.toString());
            throw new DataAccessException(he);
        }
        return;
    }

}
