package com.softslate.commerce.daos.order;

import java.util.ArrayList;
import java.util.Calendar;
import java.util.Collection;
import java.util.HashMap;
import java.util.Iterator;
import java.util.LinkedHashSet;
import java.util.List;
import java.util.Map;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.hibernate.Criteria;
import org.hibernate.HibernateException;
import org.hibernate.Query;
import org.hibernate.Session;
import org.hibernate.criterion.Restrictions;
import org.hibernate.type.IntegerType;
import org.hibernate.type.StringType;

import com.softslate.commerce.businessobjects.core.BusinessObjectFactory;
import com.softslate.commerce.businessobjects.core.User;
import com.softslate.commerce.businessobjects.order.Order;
import com.softslate.commerce.businessobjects.order.OrderDelivery;
import com.softslate.commerce.businessobjects.order.OrderItem;
import com.softslate.commerce.businessobjects.order.OrderNumber;
import com.softslate.commerce.businessobjects.order.OrderProcessor;
import com.softslate.commerce.businessobjects.order.StatesAndCountries;
import com.softslate.commerce.businessobjects.product.Discount;
import com.softslate.commerce.daos.core.BaseAdminGatewayDAOHibernate;
import com.softslate.commerce.daos.core.DataAccessException;
import com.softslate.commerce.daos.product.ProductGatewayDAO;
import com.softslate.commerce.daos.product.SKUGatewayDAO;

/**
 * Generic DAO that runs miscellaneous queries related to orders.
 * 
 * @author David Tobey
 */
public class OrderGatewayDAOHibernate extends BaseAdminGatewayDAOHibernate implements OrderGatewayDAO {

	static Log log = LogFactory.getLog(OrderGatewayDAOHibernate.class);

	public StatesAndCountries loadStatesAndCountries(StatesAndCountries statesAndCountries) throws DataAccessException {
		Session session = getDaoFactory().startSession();
		try {
			Query q = getNamedQuery(session, "country.selectAllActive");
			statesAndCountries.setCountries(new LinkedHashSet(q.list()));
			Query q2 = getNamedQuery(session, "state.selectAllActive");
			statesAndCountries.setStates(new LinkedHashSet(q2.list()));
		} catch (HibernateException he) {
			if (log.isErrorEnabled())
				log.error("Hibernate error: " + he.toString());
			throw new DataAccessException(he);
		}
		return statesAndCountries;
	}

	public Collection loadDashboardStatistics() throws DataAccessException {
		Collection stats = new ArrayList();
		Session session = getDaoFactory().startSession();
		try {
			Query q = getNamedQuery(session, "order.selectRecentActivity");
			// Last 24 hours
			Calendar start = Calendar.getInstance();
			start.add(Calendar.HOUR_OF_DAY, -24);
			q.setString("startDate", formatDateTime(new java.util.Date(start.getTimeInMillis())));
			Object[] o = (Object[]) q.uniqueResult();
			Map m = new HashMap();
			m.put("numberOfOrders", o[0]);
			m.put("totalRevenue", o[1]);
			m.put("numberOfProducts", o[2]);
			stats.add(m);
			// Last 7 days
			start = Calendar.getInstance();
			start.add(Calendar.DATE, -7);
			q.setString("startDate", formatDateTime(new java.util.Date(start.getTimeInMillis())));
			o = (Object[]) q.uniqueResult();
			Map m1 = new HashMap();
			m1.put("numberOfOrders", o[0]);
			m1.put("totalRevenue", o[1]);
			m1.put("numberOfProducts", o[2]);
			stats.add(m1);
			// Last 30 days
			start = Calendar.getInstance();
			start.add(Calendar.DATE, -30);
			q.setString("startDate", formatDateTime(new java.util.Date(start.getTimeInMillis())));
			o = (Object[]) q.uniqueResult();
			Map m2 = new HashMap();
			m2.put("numberOfOrders", o[0]);
			m2.put("totalRevenue", o[1]);
			m2.put("numberOfProducts", o[2]);
			stats.add(m2);
			// All time
			Query q2 = getNamedQuery(session, "order.selectAllActivity");
			o = (Object[]) q2.uniqueResult();
			Map m3 = new HashMap();
			m3.put("numberOfOrders", o[0]);
			m3.put("totalRevenue", o[1]);
			m3.put("numberOfProducts", o[2]);
			stats.add(m3);
		} catch (HibernateException he) {
			if (log.isErrorEnabled())
				log.error("Hibernate error: " + he.toString());
			throw new DataAccessException(he);
		}
		return stats;
	}

	public Collection loadIncompleteOrdersStatistics() throws DataAccessException {
		Collection stats = new ArrayList();
		Session session = getDaoFactory().startSession();
		try {
			Query q = getNamedQuery(session, "order.selectRecentIncomplete");
			// Last 30 minutes
			Calendar start = Calendar.getInstance();
			start.add(Calendar.MINUTE, -30);
			q.setString("startDate", formatDateTime(new java.util.Date(start.getTimeInMillis())));
			Object[] o = (Object[]) q.uniqueResult();
			Map m = new HashMap();
			m.put("numberOfOrders", o[0]);
			m.put("totalRevenue", o[1]);
			m.put("numberOfProducts", o[2]);
			stats.add(m);
			// Last 24 hours
			start = Calendar.getInstance();
			start.add(Calendar.HOUR_OF_DAY, -24);
			q.setString("startDate", formatDateTime(new java.util.Date(start.getTimeInMillis())));
			o = (Object[]) q.uniqueResult();
			m = new HashMap();
			m.put("numberOfOrders", o[0]);
			m.put("totalRevenue", o[1]);
			m.put("numberOfProducts", o[2]);
			stats.add(m);
			// Last 7 days
			start = Calendar.getInstance();
			start.add(Calendar.DATE, -7);
			q.setString("startDate", formatDateTime(new java.util.Date(start.getTimeInMillis())));
			o = (Object[]) q.uniqueResult();
			Map m1 = new HashMap();
			m1.put("numberOfOrders", o[0]);
			m1.put("totalRevenue", o[1]);
			m1.put("numberOfProducts", o[2]);
			stats.add(m1);
			// Last 30 days
			start = Calendar.getInstance();
			start.add(Calendar.DATE, -30);
			q.setString("startDate", formatDateTime(new java.util.Date(start.getTimeInMillis())));
			o = (Object[]) q.uniqueResult();
			Map m2 = new HashMap();
			m2.put("numberOfOrders", o[0]);
			m2.put("totalRevenue", o[1]);
			m2.put("numberOfProducts", o[2]);
			stats.add(m2);
			// All time
			Query q2 = getNamedQuery(session, "order.selectAllIncomplete");
			o = (Object[]) q2.uniqueResult();
			Map m3 = new HashMap();
			m3.put("numberOfOrders", o[0]);
			m3.put("totalRevenue", o[1]);
			m3.put("numberOfProducts", o[2]);
			stats.add(m3);
		} catch (HibernateException he) {
			if (log.isErrorEnabled())
				log.error("Hibernate error: " + he.toString());
			throw new DataAccessException(he);
		}
		return stats;
	}

	public int deleteIncompleteOrders(boolean commit, Map parameters)
			throws DataAccessException {
		if (getSettings().getValue("decrementBehavior") != null
				&& getSettings().getValue("decrementBehavior").equals("onAddingToCart")) {
			return deleteIncompleteOrdersWhileIncrementingInventory(commit, parameters);
		} else {
			Session session = getDaoFactory().startSession();
			try {
				Query q = getNamedQuery(session, "order.deleteIncompleteOrders");
				Calendar start = Calendar.getInstance();
				start.add(Calendar.DATE, (-1 * Integer.parseInt((String) parameters.get("olderThanDays"))));
				start.add(Calendar.HOUR_OF_DAY, (-1 * Integer.parseInt((String) parameters.get("olderThanHours"))));
				q.setString("startDate", formatDateTime(new java.util.Date(start.getTimeInMillis())));
				int updateCount = q.executeUpdate();
				if (commit) {
					getDaoFactory().commitTransaction();
				}
				return updateCount;
			} catch (Exception e) {
				if (log.isErrorEnabled())
					log.error("Order deletion error: " + e.toString());
				throw new DataAccessException(e);
			}
		}
	}

	protected int deleteIncompleteOrdersWhileIncrementingInventory(boolean commit, Map parameters)
			throws DataAccessException {
		Session session = getDaoFactory().startSession();
		try {
			OrderDAO orderDAO = (OrderDAO) getDaoFactory().createDAO(
					"orderDAOImplementer");
			Query q = getNamedQuery(session, "order.selectIncompleteOrders");
			Calendar start = Calendar.getInstance();
			start.add(Calendar.DATE, (-1 * Integer.parseInt((String) parameters
					.get("olderThanDays"))));
			start.add(Calendar.HOUR_OF_DAY, (-1 * Integer
					.parseInt((String) parameters.get("olderThanHours"))));
			q.setString("startDate", formatDateTime(new java.util.Date(start
					.getTimeInMillis())));
			List orders = q.list();
			Iterator i = orders.iterator();
			while (i.hasNext()) {
				Order o = (Order) i.next();
				if (getSettings().getValue("decrementBehavior") != null
						&& getSettings().getValue("decrementBehavior").equals(
								"onAddingToCart")) {
					incrementOrderSKUs(o);
				}
				orderDAO.setOrder(o);
				orderDAO.deleteOrder();
			}
			int updateCount = orders.size();
			if (commit) {
				getDaoFactory().commitTransaction();
			}
			return updateCount;
		} catch (Exception e) {
			if (log.isErrorEnabled())
				log.error("Order deletion error: " + e.toString());
			throw new DataAccessException(e);
		}
	}

	public void insertOrderAndDelivery(Order order, OrderDelivery orderDelivery) throws Exception {
		OrderDAO orderDAO =  getDaoFactory().createDAO(OrderDAO.class);
		orderDAO.setOrder(order);
		// Insert the order and its dependants
		if (log.isInfoEnabled())
			log.info("About to insert new order.");
		orderDAO.insertOrder();
		getDaoFactory().commitTransaction();
		if (order != null) {
			getDaoFactory().getSession().evict(order);
		}
	}

	public Collection getProductInfoFromCodes(String[] productCodeArray) throws Exception {
		if (log.isInfoEnabled())
			log.info("Getting product info from codes.");
		Session session = getDaoFactory().startSession();
		try {
			Query q = getNamedQuery(session, "order.productCodes");
			q.setCacheable(true);
			q.setParameterList("productCodes", productCodeArray, StringType.INSTANCE);
			return new LinkedHashSet(q.list());
		} catch (HibernateException he) {
			if (log.isErrorEnabled())
				log.error("Hibernate error: " + he.toString());
			throw new DataAccessException(he);
		}
	}

	public void processOrderItems(User user, Collection newOrderItems, Map parameters) throws Exception {
		if (log.isInfoEnabled())
			log.info("Processing order items.");
		// Set up the database connection and run queries within a try/catch.
		// Decrement SKUs
		if (log.isDebugEnabled())
			log.debug("Decrementing SKUs.");
		Collection skusToDecrement = (Collection) parameters.get("skusToDecrement");
		if (skusToDecrement != null && !skusToDecrement.isEmpty()) {
			decrementSKUs(false, skusToDecrement);
		}
		// Deactivate products
		if (log.isDebugEnabled())
			log.debug("Deactivating products.");
		Collection productsToDeactivate = (Collection) parameters.get("productsToDeactivate");
		if (productsToDeactivate != null && !productsToDeactivate.isEmpty()) {
			deactivateProducts(false, productsToDeactivate);
		}
		// Add the OrderItems and update the order and order delivery
		if (log.isInfoEnabled())
			log.info("Adding cart items and updating totals for order id " + user.getOrder().getOrderID());
		updateOrderAndDelivery(false, user.getOrder(), user.getOrder().firstDelivery());
		getDaoFactory().commitTransaction();
		if (user.getOrder() != null) {
			getDaoFactory().getSession().evict(user.getOrder());
		}
	}

	public void decrementSKUs(boolean commit, Collection skusToDecrement) throws Exception {
		Iterator i = skusToDecrement.iterator();
		// Loop through the skus, decrementing each one.
		while (i.hasNext()) {
			// Decrement the SKU in the database.
			Map sku = (Map) i.next();
			String skuID = (String) sku.get("skuID");
			OrderItem orderItem = (OrderItem) sku.get("orderItem");
			int quantity = orderItem.getQuantity();
			SKUGatewayDAO skuDAO =  getDaoFactory().createDAO(SKUGatewayDAO.class);
			skuDAO.decrementSKU(Integer.parseInt(skuID), quantity);
		}
		if (commit) {
			getDaoFactory().commitTransaction();
		}
	}

	public void incrementSKUs(boolean commit, OrderItem orderItem) throws Exception {
		SKUGatewayDAO skuDAO =  getDaoFactory().createDAO(SKUGatewayDAO.class);
		if (orderItem != null && orderItem.getDecrementedSKUs() != null && orderItem.getDecrementedSKUs().length() > 0) {
			if (log.isInfoEnabled())
				log.info("Incrementing SKUs: " + orderItem.getDecrementedSKUs());
			String[] ids = orderItem.getDecrementedSKUs().split(",");
			for (int i = 0; i < ids.length; i++) {
				String id = ids[i].trim();
				String[] skuInfo = id.split(":");
				if (skuInfo.length > 0) {
					skuDAO.incrementSKU(Integer.parseInt(skuInfo[0]), orderItem.getQuantity());
				}
			}
		}
		if (commit) {
			getDaoFactory().commitTransaction();
		}
	}

	/**
	 * Increments all the SKUs associated with the order items in a given order
	 * by the quantity of the item. Used to refresh the SKU inventory levels
	 * when incomplete orders are deleted, if the inventory decrement behavior
	 * is set to decrement upon adding items to the cart.
	 * 
	 * @param o
	 *            The <code>Order</code> being processed.
	 * @throws Exception
	 */
	public void incrementOrderSKUs(Order o) throws Exception {
		SKUGatewayDAO skuDAO =  getDaoFactory().createDAO(SKUGatewayDAO.class);
		if (o.getOrderDeliveryCollection() != null) {
			Iterator j = o.getOrderDeliveryCollection().iterator();
			while (j.hasNext()) {
				OrderDelivery od = (OrderDelivery) j.next();
				if (od.getOrderItemCollection() != null) {
					Iterator k = od.getOrderItemCollection().iterator();
					while (k.hasNext()) {
						OrderItem oi = (OrderItem) k.next();
						if (oi.getDecrementedSKUs() != null && oi.getDecrementedSKUs().length() > 0) {
							String[] ids = oi.getDecrementedSKUs().split(",");
							for (int m = 0; m < ids.length; m++) {
								String id = ids[m].trim();
								String[] skuInfo = id.split(":");
								if (skuInfo.length > 0) {
									try {
										skuDAO.incrementSKU(Integer.parseInt(skuInfo[0]), oi.getQuantity());
									} catch (NumberFormatException nfe) {
										log.error(nfe.toString(), nfe);
									}
								}
							}
						}
					}
				}
			}
		}
	}

	public void deactivateProducts(boolean commit, Collection productsToDeactivate) throws Exception {
		if (log.isDebugEnabled())
			log.debug("Deactivating products.");
		ProductGatewayDAO dao =  getDaoFactory().createDAO(ProductGatewayDAO.class);
		dao.updateProductsActive(productsToDeactivate, false);
	}

	public void removeOrderItem(Order order, OrderDelivery orderDelivery, OrderItem orderItem) throws Exception {
		// Delete the order item
		if (log.isInfoEnabled())
			log.info("Deleting order item id: " + orderItem.getOrderItemID());
		// Increment the SKUs that were decremented previously, if any
		incrementSKUs(false, orderItem);

		// Commented out because in deleting the item, the item is loaded into
		// the Hibernate session and its parent order is too.
		// This caused a "NonUniqueObjectException" below with
		// updateOrderAndDelivery because that method loads the order again into
		// getUser().getOrder().
		// OrderItemDAO orderItemDAO = (OrderItemDAO) getDaoFactory().createDAO(
		// "orderItemDAOImplementer");
		// orderItemDAO.setOrderItem(orderItem);
		// orderItemDAO.deleteOrderItem();

		// Update the order and order delivery
		if (log.isInfoEnabled())
			log.info("Updating totals for order id " + order.getOrderID());
		updateOrderAndDelivery(false, order, orderDelivery);
		getDaoFactory().commitTransaction();
		if (order != null) {
			getDaoFactory().getSession().evict(order);
		}
	}

	public void updateOrderAndDelivery(boolean commit, Order order, OrderDelivery orderDelivery) throws Exception {
		if (log.isInfoEnabled())
			log.info("About to update order and delivery.");

		// Update the order and all its dependants
		OrderDAO orderDAO =  getDaoFactory().createDAO(OrderDAO.class);
		orderDAO.setOrder(order);
		orderDAO.updateOrder();
		if (commit) {
			getDaoFactory().commitTransaction();
			if (order != null) {
				getDaoFactory().getSession().evict(order);
			}
		}
	}

	public Map loadOrdersAndCount(Map parameters) throws Exception {
		return loadItemsAndCount(parameters);
	}

	public Map editDeleteOrders(Map parameters, BusinessObjectFactory bof) throws Exception {
		return editDeleteItems(parameters, bof);
	}

	public Criteria processCommonCriteria(Criteria crit, Map parameters) {
		String workWithOrders = (String) parameters.get("workWithOrders");
		if (workWithOrders == null) {
			crit = super.processCommonCriteria(crit, parameters);
			crit.add(Restrictions.isNotNull("completed"));
		} else if (workWithOrders.equals("All Completed")) {
			crit = super.processCommonCriteria(crit, parameters);
			crit.add(Restrictions.isNotNull("completed"));
		} else if (workWithOrders.equals("Incomplete")) {
			crit = super.processCommonCriteria(crit, parameters);
			crit.add(Restrictions.isNull("completed"));
		} else {
			crit = super.processCommonCriteria(crit, parameters);
			crit.add(Restrictions.isNotNull("completed"));
			crit.add(Restrictions.eq("status", workWithOrders));
		}
		return crit;
	}

	public Map postChangingItemEvent(Object item, Map parameters, BusinessObjectFactory bof) throws Exception {
		OrderProcessor processor =  bof.createObject(OrderProcessor.class);
		Map results = processor.postChangingOrderEvent((Order) item, parameters);
		return results;
	}

	public Map postChangedItemEvent(Object item, Map parameters, BusinessObjectFactory bof) throws Exception {
		OrderProcessor processor =  bof.createObject(OrderProcessor.class);
		Map results = processor.postChangedOrderEvent((Order) item, parameters);
		return results;
	}

	public void processOrderComplete(Order order, Map parameters) throws Exception {
		if (log.isInfoEnabled())
			log.info("Processing order on completion.");
		if (parameters != null) {
			// Decrement SKUs
			if (log.isDebugEnabled())
				log.debug("Decrementing SKUs.");
			Collection skusToDecrement = (Collection) parameters.get("skusToDecrement");
			if (skusToDecrement != null && !skusToDecrement.isEmpty()) {
				decrementSKUs(false, skusToDecrement);
			}
			// Deactivate products
			if (log.isDebugEnabled())
				log.debug("Deactivating products.");
			Collection productsToDeactivate = (Collection) parameters.get("productsToDeactivate");
			if (productsToDeactivate != null && !productsToDeactivate.isEmpty()) {
				deactivateProducts(false, productsToDeactivate);
			}
		}
	}

	public void insertOrderNumber(OrderNumber orderNumber, boolean commit) throws DataAccessException {
		Session session = getDaoFactory().startSession();
		try {
			session.save(orderNumber);
			if (commit) {
				getDaoFactory().commitTransaction();
			}
		} catch (HibernateException he) {
			if (log.isErrorEnabled())
				log.error("Hibernate error: " + he.toString());
			throw new DataAccessException(he);
		}
		return;
	}

	public void markOrderComplete(Order order, Map parameters) throws Exception {
		OrderDAO orderDAO =  getDaoFactory().createDAO(OrderDAO.class);
		orderDAO.setOrder(order);
		// Update the order
		orderDAO.updateOrder();
		// Commit the transaction
		getDaoFactory().commitTransaction();
		if (log.isInfoEnabled())
			log.info("Committed transaction.");
	}

	public int deleteOrderDiscounts(boolean commit, int orderID) throws DataAccessException {
		Session session = getDaoFactory().startSession();
		try {
			Query q = getNamedQuery(session, "order.deleteOrderDiscounts");
			q.setInteger("orderID", orderID);
			int updateCount = q.executeUpdate();
			if (commit) {
				getDaoFactory().commitTransaction();
			}
			return updateCount;
		} catch (HibernateException he) {
			if (log.isErrorEnabled())
				log.error("Hibernate error: " + he.toString());
			throw new DataAccessException(he);
		}
	}

	public Collection getDiscountsBySkuIDs(Collection skuIDs) throws Exception {
		Session session = getDaoFactory().startSession();
		try {
			Query q = getNamedQuery(session, "order.discountsBySkuIDs");
			q.setParameterList("skuIDs", skuIDs, IntegerType.INSTANCE);

			// Don't cache the discounts since info that applies across order
			// items is stored in each
			Collection discounts = new LinkedHashSet(q.list());
			Iterator id = discounts.iterator();
			while (id.hasNext()) {
				Discount discount = (Discount) id.next();
				if (discount != null) {
					session.evict(discount);
				}
			}
			return discounts;
		} catch (HibernateException he) {
			if (log.isErrorEnabled())
				log.error("Hibernate error: " + he.toString());
			throw new DataAccessException(he);
		}
	}

	public Double getPreviousOrderTotal(int customerID) throws Exception {
		Session session = getDaoFactory().startSession();
		try {
			Query q = getNamedQuery(session, "order.previousOrderTotal");
			q.setInteger("customerID", customerID);
			return (Double) q.uniqueResult();
		} catch (HibernateException he) {
			if (log.isErrorEnabled())
				log.error("Hibernate error: " + he.toString());
			throw new DataAccessException(he);
		}
	}

	public Integer getTimesUsedByCustomer(int discountID, int customerID) throws Exception {
		Session session = getDaoFactory().startSession();
		try {
			Query q = getNamedQuery(session, "order.timesUsedByCustomer");
			q.setInteger("customerID", customerID);
			q.setInteger("discountID", discountID);
			return new Integer(((Long) q.uniqueResult()).intValue());
		} catch (HibernateException he) {
			if (log.isErrorEnabled())
				log.error("Hibernate error: " + he.toString());
			throw new DataAccessException(he);
		}
	}

	public Collection getGlobalDiscounts() throws Exception {
		Session session = getDaoFactory().startSession();
		try {
			Query q = getNamedQuery(session, "order.globalDiscounts");
			q.setCacheable(true);
			return new LinkedHashSet(q.list());
		} catch (HibernateException he) {
			if (log.isErrorEnabled())
				log.error("Hibernate error: " + he.toString());
			throw new DataAccessException(he);
		}
	}

	public Collection getGlobalDiscounts(Collection couponCodes) throws Exception {
		if (couponCodes == null || couponCodes.isEmpty()) {
			return getGlobalDiscounts();
		}
		Session session = getDaoFactory().startSession();
		try {
			Query q = getNamedQuery(session, "order.globalDiscountsAndCoupons");
			q.setParameterList("couponCodes", couponCodes);
			q.setCacheable(true);
			return new LinkedHashSet(q.list());
		} catch (HibernateException he) {
			if (log.isErrorEnabled())
				log.error("Hibernate error: " + he.toString());
			throw new DataAccessException(he);
		}
	}

	public Collection loadShippingDiscounts() throws Exception {
		Session session = getDaoFactory().startSession();
		try {
			Query q = getNamedQuery(session, "order.shippingDiscounts");
			q.setCacheable(true);
			return new LinkedHashSet(q.list());
		} catch (HibernateException he) {
			if (log.isErrorEnabled())
				log.error("Hibernate error: " + he.toString());
			throw new DataAccessException(he);
		}
	}

	public Collection loadShippingDiscounts(Collection couponCodes) throws Exception {
		if (couponCodes == null || couponCodes.isEmpty()) {
			return loadShippingDiscounts();
		}
		Session session = getDaoFactory().startSession();
		try {
			Query q = getNamedQuery(session, "order.shippingDiscountsAndCoupons");
			q.setParameterList("couponCodes", couponCodes);
			q.setCacheable(true);
			return new LinkedHashSet(q.list());
		} catch (HibernateException he) {
			if (log.isErrorEnabled())
				log.error("Hibernate error: " + he.toString());
			throw new DataAccessException(he);
		}
	}

	public Collection loadGiftCertificates(Collection couponCodes) throws Exception {
		if (couponCodes == null || couponCodes.isEmpty()) {
			return new LinkedHashSet();
		}
		Session session = getDaoFactory().startSession();
		try {
			Query q = getNamedQuery(session, "order.giftCertificates");
			q.setParameterList("couponCodes", couponCodes);
			q.setCacheable(true);
			return new LinkedHashSet(q.list());
		} catch (HibernateException he) {
			if (log.isErrorEnabled())
				log.error("Hibernate error: " + he.toString());
			throw new DataAccessException(he);
		}
	}

	public Collection loadExport(String startDate, String endDate, boolean allOrders, int firstID, int lastID)
			throws Exception {
		if (log.isInfoEnabled())
			log.info("Exporting orders within an id range: " + firstID + "-" + lastID);
		Session session = getDaoFactory().startSession();
		try {
			Query q;
			if (allOrders) {
				q = getNamedQuery(session, "orders.exportSelectAll");
			} else {
				q = getNamedQuery(session, "orders.exportSelectCompleted");
			}
			q.setString("startDate", startDate);
			q.setString("endDate", endDate);
			q.setInteger("firstID", firstID);
			q.setInteger("lastID", lastID);
			List orders = q.list();
			String[] returnFields = q.getReturnAliases();
			if (log.isDebugEnabled())
				log.debug("returnFields: " + returnFields);
			Collection results = new ArrayList();
			if (null != orders)
				for (int i = 0; i < orders.size(); i++) {
					Object[] row = (Object[]) orders.get(i);
					Map resultRow = new HashMap();
					for (int j = 0; j < row.length; j++) {
						if (row[j] != null) {

							// Decrypt the credit card number
							if (returnFields[j].equals("payment_creditCardNumber")) {
								String decrypted = getDAOUtils().getDecrypted(row[j].toString(),
										(String) getSettings().getValue("paymentEncryptionType"),
										getAppSettings().getProperty("twoWayKeyFile"));
								resultRow.put(returnFields[j], decrypted);
							} else {
								resultRow.put(returnFields[j], row[j].toString());
							}
						}
					}
					results.add(resultRow);
				}
			return results;
		} catch (HibernateException he) {
			if (log.isErrorEnabled())
				log.error("Hibernate error: " + he.toString());
			throw new DataAccessException(he);
		}
	}

	public Collection loadSalesByProductReport(String startDate, String endDate, Integer[] productIDs)
			throws DataAccessException {
		Session session = getDaoFactory().startSession();
		try {
			Query q = getNamedQuery(session, "order.salesByProductReport");
			q.setString("startDate", startDate);
			q.setString("endDate", endDate);
			q.setParameterList("productIDs", productIDs);
			return q.list();
		} catch (HibernateException he) {
			if (log.isErrorEnabled())
				log.error("Hibernate error: " + he.toString());
			throw new DataAccessException(he);
		}
	}

	public Collection loadMonthlySalesReport(String startDate, String endDate) throws DataAccessException {
		Session session = getDaoFactory().startSession();
		try {
			Query q = getNamedQuery(session, "order.monthlySalesReport");
			q.setString("startDate", startDate);
			q.setString("endDate", endDate);
			return q.list();
		} catch (HibernateException he) {
			if (log.isErrorEnabled())
				log.error("Hibernate error: " + he.toString());
			throw new DataAccessException(he);
		}
	}

	public int loadMaxOrderID() throws Exception {
		Session session = getDaoFactory().startSession();
		try {
			Query q = getNamedQuery(session, "order.maxOrderID");
			Integer maxNum = (Integer) q.uniqueResult();
			if (maxNum == null) {
				return 0;
			}
			return maxNum.intValue();
		} catch (HibernateException he) {
			if (log.isErrorEnabled())
				log.error("Hibernate error: " + he.toString());
			throw new DataAccessException(he);
		}
	}

	public Collection loadIncompleteOrdersWithPayments(Map parameters) throws DataAccessException {
		Session session = getDaoFactory().startSession();
		try {
			Query q = getNamedQuery(session, "order.selectIncompleteOrdersWithPayments");
			Calendar start = Calendar.getInstance();
			start.add(Calendar.DATE, (-1 * Integer.parseInt((String) parameters.get("olderThanDays"))));
			start.add(Calendar.HOUR_OF_DAY, (-1 * Integer.parseInt((String) parameters.get("olderThanHours"))));
			q.setString("endDate", formatDateTime(new java.util.Date(start.getTimeInMillis())));
			return q.list();
		} catch (Exception e) {
			if (log.isErrorEnabled())
				log.error("Hibernate error: " + e.toString());
			throw new DataAccessException(e);
		}
	}

	public Collection<Order> loadCompletedOrdersByDateRange(String startDate, String endDate,
			Collection<String> ignoreStatuses) throws DataAccessException {
		Session session = getDaoFactory().startSession();
		if (ignoreStatuses == null || ignoreStatuses.isEmpty()) {
			ignoreStatuses = new ArrayList<String>();
			ignoreStatuses.add("DUMMYVALUE");
		}
		try {
			Query q = getNamedQuery(session, "order.selectCompletedOrdersByDateRange");
			q.setString("startDate", startDate);
			q.setString("endDate", endDate);
			q.setParameterList("ignoreStatuses", ignoreStatuses);
			return q.list();
		} catch (Exception e) {
			if (log.isErrorEnabled())
				log.error("Hibernate error: " + e.toString());
			throw new DataAccessException(e);
		}
	}

	public Collection<Order> loadAbandonedCartOrders(String startDate, String endDate, String prevOrdersDate)
			throws DataAccessException {
		Session session = getDaoFactory().startSession();
		try {
			Query q = getNamedQuery(session, "order.loadAbandonedCartOrders");
			q.setString("startDate", startDate);
			q.setString("endDate", endDate);
			q.setString("prevDate", prevOrdersDate);
			return q.list();
		} catch (Exception e) {
			if (log.isErrorEnabled())
				log.error("Hibernate error: " + e.toString());
			throw new DataAccessException(e);
		}
	}

	public Collection<Order> loadCompletedOrdersByLastModifiedAndStatuses(String startDate, String endDate,
			Collection<String> currentStatuses) throws DataAccessException {
		Session session = getDaoFactory().startSession();
		if (currentStatuses == null || currentStatuses.isEmpty()) {
			currentStatuses = new ArrayList<String>();
			currentStatuses.add("DUMMYVALUE");
		}
		try {
			Query q = getNamedQuery(session, "order.loadCompletedOrdersByLastModifiedAndStatuses");
			q.setString("startDate", startDate);
			q.setString("endDate", endDate);
			q.setParameterList("currentStatuses", currentStatuses);
			return q.list();
		} catch (Exception e) {
			if (log.isErrorEnabled())
				log.error("Hibernate error: " + e.toString());
			throw new DataAccessException(e);
		}
	}

}
