package com.softslate.commerce.daos.product;

import java.util.Collection;
import java.util.LinkedHashSet;
import java.util.Map;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.hibernate.HibernateException;
import org.hibernate.Query;
import org.hibernate.Session;

import com.softslate.commerce.businessobjects.core.BusinessObjectFactory;
import com.softslate.commerce.businessobjects.product.SKU;
import com.softslate.commerce.businessobjects.product.SKUProcessor;
import com.softslate.commerce.daos.core.BaseAdminGatewayDAOHibernate;
import com.softslate.commerce.daos.core.DataAccessException;

/**
 * Generic DAO that runs miscellaneous queries related to skus.
 * 
 * @author David Tobey
 */
public class SKUGatewayDAOHibernate extends BaseAdminGatewayDAOHibernate implements SKUGatewayDAO {

	static Log log = LogFactory.getLog(SKUGatewayDAOHibernate.class);

	public Map loadSKUsAndCount(Map parameters) throws Exception {
		return loadItemsAndCount(parameters);
	}

	public Map editDeleteSKUs(Map parameters, BusinessObjectFactory bof) throws Exception {
		Map result = editDeleteItems(parameters, bof);
		evictCache();
		return result;
	}

	public Map postChangingItemEvent(Object item, Map parameters, BusinessObjectFactory bof) throws Exception {
		SKUProcessor processor = bof.createObject(SKUProcessor.class);
		Map results = processor.postChangingSKUEvent((SKU) item, parameters);
		return results;
	}

	public void decrementSKU(int skuID, int quantity) throws Exception {
		decrementSKU(false, skuID, quantity);
	}

	public void decrementSKU(boolean commit, int skuID, int quantity) throws Exception {
		Session session = getDaoFactory().startSession();
		try {
			Query q = getNamedQuery(session, "sku.decrementSKU");
			q.setInteger("skuID", skuID);
			q.setInteger("quantity", quantity);
			q.executeUpdate();
			if (commit) {
				getDaoFactory().commitTransaction();
			}
		} catch (HibernateException he) {
			if (log.isErrorEnabled())
				log.error("Hibernate error: " + he.toString());
			throw new DataAccessException(he);
		}
	}

	public void incrementSKU(int skuID, int quantity) throws Exception {
		incrementSKU(false, skuID, quantity);
	}

	public void incrementSKU(boolean commit, int skuID, int quantity) throws Exception {
		Session session = getDaoFactory().startSession();
		try {
			Query q = getNamedQuery(session, "sku.incrementSKU");
			q.setInteger("skuID", skuID);
			q.setInteger("quantity", quantity);
			q.executeUpdate();
			if (commit) {
				getDaoFactory().commitTransaction();
			}
		} catch (HibernateException he) {
			if (log.isErrorEnabled())
				log.error("Hibernate error: " + he.toString());
			throw new DataAccessException(he);
		}
	}

	public void deleteOrphanedSKUs(boolean commit) throws Exception {
		Session session = getDaoFactory().startSession();
		try {
			Query q = getNamedQuery(session, "sku.deleteOrphanedSKUs");
			q.executeUpdate();
			if (commit) {
				getDaoFactory().commitTransaction();
			}
		} catch (HibernateException he) {
			if (log.isErrorEnabled())
				log.error("Hibernate error: " + he.toString());
			throw new DataAccessException(he);
		}
	}

	public int loadMaxSKUID() throws Exception {
		Session session = getDaoFactory().startSession();
		try {
			Query q = getNamedQuery(session, "product.maxSKUID");
			Integer maxNum = (Integer) q.uniqueResult();
			if (maxNum == null) {
				return 0;
			}
			return maxNum.intValue();
		} catch (HibernateException he) {
			if (log.isErrorEnabled())
				log.error("Hibernate error: " + he.toString());
			throw new DataAccessException(he);
		}
	}

	public Collection loadAllSKUsInRange(int firstID, int lastID) throws Exception {
		if (log.isInfoEnabled())
			log.info("Loading skus within an id range: " + firstID + "-" + lastID);
		Collection items = null;
		Session session = getDaoFactory().startSession();
		try {
			Query q = getNamedQuery(session, "sku.selectAllInRange");
			q.setInteger("firstID", firstID);
			q.setInteger("lastID", lastID);
			items = new LinkedHashSet(q.list());
		} catch (HibernateException he) {
			if (log.isErrorEnabled())
				log.error("Hibernate error: " + he.toString());
			throw new DataAccessException(he);
		}
		return items;
	}
}
