package com.softslate.commerce.installer;

import java.io.File;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.log4j.PropertyConfigurator;
import org.apache.struts.action.ActionErrors;
import org.apache.struts.action.ActionForm;
import org.apache.struts.action.ActionForward;
import org.apache.struts.action.ActionMapping;
import org.apache.struts.action.ActionMessage;
import org.apache.struts.action.ActionMessages;
import org.apache.struts.action.DynaActionForm;

import com.softslate.commerce.customer.core.ActionUtils;
import com.softslate.commerce.customer.core.ActionUtilsImpl;
import com.softslate.commerce.customer.core.BaseAction;

/**
 * 
 * @author David Tobey
 */
public class ConfigureLogFileAction extends BaseAction {

	static Log log = LogFactory.getLog(ConfigureLogFileAction.class);

	public ActionForward execute(ActionMapping mapping, ActionForm form,
			HttpServletRequest request, HttpServletResponse response)
			throws Exception {
		if (log.isDebugEnabled())
			log.debug("Starting action.");
		ActionErrors errors = new ActionErrors();
		ActionMessages messages = new ActionMessages();
		DynaActionForm dynaForm = (DynaActionForm) form;
		boolean error = false;

		// Save the settings to the log4j.properties file.
		ActionUtils au = new ActionUtilsImpl();

		// Check that the log file directory submitted in the form exists. If it
		// does not, don't treat it as an error but display a message.
		String logFileDirectory = (String) dynaForm.get("logFileDirectory");
		if (logFileDirectory == null || logFileDirectory.length() == 0
				|| !(new File(logFileDirectory).isDirectory())) {
			messages.add(ActionMessages.GLOBAL_MESSAGE, new ActionMessage(
					"installer.logFileDirectoryError", logFileDirectory));
		} else {

			// Escape backslashes in case it's a windows path.
			logFileDirectory = logFileDirectory.replaceAll("\\\\",
					"\\\\\\\\\\\\\\\\");

			// Find the log4j.properties file
			String log4jContents = null;
			String log4jPropertiesFile = "log4j.properties";
			if (System.getProperty("log4j.configuration") != null) {
				log4jPropertiesFile = System.getProperty("log4j.configuration");
			}
			String log4jPath = getServlet().getServletContext().getRealPath(
					"/WEB-INF/classes/" + log4jPropertiesFile);
			if (log4jPath == null) {
				errors.add(ActionMessages.GLOBAL_MESSAGE, new ActionMessage(
						"installer.warError", log4jPath));
				error = true;
			} else {
				try {

					// Load the contents, insert the directory and save the file
					log4jContents = au.readFile(log4jPath);
					log4jContents = log4jContents.replaceAll(
							"\\$\\{catalina.home\\}\\/logs\\/",
							logFileDirectory);
					if (log.isDebugEnabled())
						log.debug("Save the settings to the "
								+ log4jPropertiesFile + " file.");
					au.writeFile(log4jPath, log4jContents);
					messages.add(ActionMessages.GLOBAL_MESSAGE,
							new ActionMessage("installer.log4jSettingsSaved",
									log4jPath));

					// In case we're in an environment with an src folder, save
					// the file to the src area too.
					String log4jPathSrc = getServlet().getServletContext()
							.getRealPath("/WEB-INF/src/" + log4jPropertiesFile);
					File log4jFileSrc = new File(log4jPathSrc);
					if (log4jContents != null && log4jFileSrc != null
							&& log4jFileSrc.exists()) {
						au.writeFile(log4jPathSrc, log4jContents);
						messages.add(ActionMessages.GLOBAL_MESSAGE,
								new ActionMessage(
										"installer.log4jSettingsSaved",
										log4jPathSrc));
					}

				} catch (Exception fnfe) {
					log.error(fnfe.toString(), fnfe);
					errors.add(ActionMessages.GLOBAL_MESSAGE,
							new ActionMessage("installer.fileNotFoundError",
									log4jPath));
					error = true;
				}

				// Reload the log4j.properties file
				if (log.isInfoEnabled())
					log.info("Attempting to reload log4j settings.");
				PropertyConfigurator.configure(log4jPath);
			}
		}

		if (!messages.isEmpty())
			addMessages(request, messages);
		if (!errors.isEmpty())
			addErrors(request, errors);
		if (log.isDebugEnabled())
			log.debug("Finished with Action.");
		if (error) {
			return mapping.findForward("error");
		}
		return mapping.findForward("success");
	}
}
